@@ -17,10 +17,10 @@ if [ $# -eq 1 ]; then
         echo "error: cannot find kernel image '$1'" 1>&2
         exit 1
     fi
-elif [ -f $PWD/linux ]; then
-    IMAGE="$PWD/linux"
-elif [ -f /mnt/linux ]; then
-    IMAGE=/mnt/linux
+elif [ -f $PWD/linux.bin ]; then
+    IMAGE="$PWD/linux.bin"
+elif [ -f /mnt/linux.bin ]; then
+    IMAGE=/mnt/linux.bin
 else
     echo "error: no kernel image supplied, nor in PWD, nor in /mnt" 1>&2
     exit 1
