@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 # Boot Disk maker for TFTP prototcol.
 # Eric Delaunay, February 1998.
 # Ben Collins, March 2000
@@ -8,10 +8,10 @@
 
 # Print a usage message and exit if the argument count is wrong.
 if [ $# != 3 ]; then
-echo "Usage: "$0" linux system.map root-image debian-version tftpimage" 1>&2
+echo "Usage: "$0" linux.bin root-image tftpimage" 1>&2
 	cat 1>&2 << EOF
 
-	linux: the Linux kernel (may be compressed).
+	linux.bin: the Linux kernel (may be compressed).
 	root-image: a compressed disk image to load in ramdisk and mount as root.
 	tftpimage: name of the image.
 EOF
@@ -38,7 +38,7 @@ done
 tmp=${tmpdir}/boot-floppies/tftpboot.$$
 make_tmpdir $tmp
 
-if [ "$arch" = arm ]; then
+if [ "$arch" = arm ] || [ "$arch" == i386 ] || [ "$arch" == mips ] || [ "$arch" == mipsel ]; then
 	cp $kernel $tmp/image
 else
 	debug "uncompressing kernel"
@@ -50,14 +50,9 @@ cp $tmp/image $tftpimage
 
 # append rootimage to the kernel
 if [ "$arch" = sparc ]; then
-	if (echo $tftpimage | grep -q sun4u); then
-		tilo "" $tftpimage $rootimage $tftpimage.tmp
-	else
-		tilo $tftpimage "" $rootimage $tftpimage.tmp
-	fi
+	tilo -o $tftpimage.tmp $tftpimage $rootimage
 	mv $tftpimage.tmp $tftpimage
-fi
-if [ "$arch" = arm ]; then
+elif [ "$arch" = arm ]; then
 	if (echo $tftpimage | grep -q cats); then
 		catsboot $tftpimage.tmp $tftpimage $rootimage
 		mv $tftpimage.tmp $tftpimage
@@ -65,6 +60,9 @@ if [ "$arch" = arm ]; then
 	if (echo $tftpimage | grep -q netwinder); then
 		cat $rootimage >>$tftpimage
 	fi
+elif [ "$arch" = "mips" -o "$arch" = "mipsel" ]; then
+		mips-specials/addinitrd $tftpimage $rootimage $tftpimage.tmp
+		mv $tftpimage.tmp $tftpimage
 fi
 
 # cleanup
