@@ -8,7 +8,7 @@ int interactive_shell(void) {
   chdir("/");
   boxSuspend();
   printf(CLEAR);
-  printf(_("You are running \"ash\", a Bourne-shell clone. The root filesystem is a\nRAM disk or floppy. The hard disk filesystems are mounted on \"/target\".\nThe editor available to you on this floppy is \"ae\". It's very small\nand easy to figure out - sorry but \"vi\" and \"emacs\" wouldn't fit.\nTo get an idea of what Unix utilities are available to you, run \"ls /bin\n/sbin /usr/bin /usr/sbin\". Use the \"exit\" command to return to the\ninstallation menu.\n"));
+  printf(_("You are running \"ash\", a Bourne-shell clone. The root filesystem is a\nRAM disk or floppy. The hard disk filesystems are mounted on \"/target\".\nThe editor available to you on this floppy is \"nano-tiny\". It's very small\nand easy to figure out - sorry but \"vi\" and \"emacs\" wouldn't fit.\nTo get an idea of what Unix utilities are available to you, run \"ls /bin\n/sbin /usr/bin /usr/sbin\". Use the \"exit\" command to return to the\ninstallation menu.\n"));
   system("/bin/sh");
   boxResume();
   chdir("/");
