@@ -2,6 +2,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <signal.h>
 #include <sys/ioctl.h>
 #include <sys/klog.h>
 #ifndef _TESTING_
@@ -9,9 +10,10 @@
 #endif
 #ifdef _TESTING_
 # define RB_AUTOBOOT 1
-reboot(int blah){}
+void reboot(int blah){}
 #endif
 #include <sys/types.h>
+#include <sys/utsname.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <asm/types.h>
@@ -32,6 +34,9 @@ char *append_opts;
 char *Arch2;
 char kernel_image_path[PATH_MAX + 1];
 char drivers_path[PATH_MAX + 1];
+char *base_cd;
+char *kernel_cd;
+int wants_cd;
 
 /* Defaults to no serial console present */
 int serialConsole = -1;
@@ -75,7 +80,6 @@ int parsefile(const char *filename, struct Arg *options,
       while (*end && !isspace(*end)) end++;
       if (!*end) lineend = 1;
       *end = '\0';
-
       a = options;
       while ( a->name != NULL ) {
 	if (a->isflag) {
@@ -124,7 +128,7 @@ int parsefile(const char *filename, struct Arg *options,
   return 0;
 }
 
-#if #cpu(alpha) || #cpu(sparc)
+#if #cpu(alpha) || #cpu(sparc) || #cpu(mips)
 static int
 parse_cpuinfo(const char *key, char *val, int len)
 {
@@ -132,7 +136,7 @@ parse_cpuinfo(const char *key, char *val, int len)
     char format[256];
     int found = 0;
 
-    snprintf(format, sizeof(format), "%s : %%%ds", key, len-1);
+    snprintf(format, sizeof(format), "%s : %%%d[A-Za-z /0-9]", key, len-1);
 
 #if defined(_EXETESTING_) || defined(_TESTING_)
     if ((cpuinfo = fopen ("testcpuinfo", "r")) != NULL)
@@ -166,11 +170,12 @@ readcmdline (void)
     { "bootkbd=",  0, &bootargs.kbd      },
     { "mouse=",    0, &bootargs.mouse    },
     { "flavor=",   0, &bootargs.flavor   },
+    { "console=",  0, &bootargs.console  },
     { "mono",      1, &bootargs.ismono   },
     { "debug",     1, &bootargs.isdebug  },
     { "verbose",   1, &bootargs.isverbose}, /* chatty mode */
     { "quiet",     1, &bootargs.isquiet  }, /* quiet mode */
-    { "cdrom",     1, &bootargs.cdrom    }, /* cdrom install */
+/*    { "cdrom",     1, &bootargs.cdrom    }, // cdrom install */
     { NULL,        0, NULL               }
   };
 
@@ -214,12 +219,16 @@ static char *get_m68k_model( void )
 #endif
 
 #if #cpu (powerpc)
+char * Arch3=NULL;
+char * Arch4=NULL;
 static char *get_powerpc_model( void )
 {
   char *model = NULL;
+  char *pmacgeneration = NULL;
   int status;
   struct Arg archoptions[] = {
     { "machine", 0, &model },
+    { "pmac-generation", 0, &pmacgeneration },
     { NULL, 0, NULL}
   };
   status = parsefile("/proc/cpuinfo", archoptions, 2, 0);
@@ -230,9 +239,36 @@ static char *get_powerpc_model( void )
   if (!model || !*model)
     fprintf( stderr, _("Unable to determine PowerPC machine type!\n") );
   if ((strncasecmp(model, "Power", 5) == 0)
-     || (strncasecmp(model, "iMac",  4) == 0)) model = "PowerMac";
+     || (strncasecmp(model, "iMac",  4) == 0)) {
+    /* Hack: if we're running a 2.4 kernel, then use the new-powermac
+       subarch */
+    if (!strncmp("2.4", kver, 3))
+      model = "NewPowerMac";
+    else
+      model = "PowerMac";
+/* 
+    FIXME
+    Okay, parsefile should be probably be fixed to do right the first time, 
+    but this works now. (parsefile doesn't seem to read to the end of 
+    the file.)
+*/
+    archoptions[0].name = "pmac-generation";
+    archoptions[0].value = &pmacgeneration;
+    status = parsefile("/proc/cpuinfo", archoptions, 2, 0);
+    if ( status )
+      ERRMSG( "Unable to determine PowerMac generation!" );
+    Arch3 = pmacgeneration;	
+    
+    archoptions[0].name = "motherboard";
+    archoptions[0].value = &Arch4;
+    status = parsefile("/proc/cpuinfo", archoptions, 2, 0);
+    if ( status )
+      ERRMSG( "Unable to determine PowerMac motherboard!" );
+    
+  }
   if (strcmp(model,"Amiga") == 0)
     model = "apus";
+
   return model;
 }
 #endif
@@ -279,7 +315,7 @@ char* get_alpha_model(void)
     int i;
     char *c;
     for (i = 0; i < ALPHA_MODEL_COUNT; i++) {
-      if (strcmp(alpha_models[i].systype, buf) == 0)
+      if (strncmp(alpha_models[i].systype, buf, strlen (alpha_models[i].systype)) == 0)
 	return strdup(alpha_models[i].model);
     }
     /* else, take the one we got and downcase it */
@@ -311,30 +347,107 @@ static char* get_sparc_model(void)
 #if #cpu(arm)
 char *Arch3;
 
+int strongarm_is_broken;
+
 static char* get_arm_model(void)
 {
   char *model = NULL;
+  char *proc = NULL;
   int status;
-  struct Arg archoptions[] = {
+  struct Arg archoptions1[] = {
     { "Hardware", 0, &model },
     { NULL, 0, NULL}
   };
-  status = parsefile("/proc/cpuinfo", archoptions, 2, 0);
+  struct Arg archoptions2[] = {
+    { "Processor", 0, &proc },
+    { NULL, 0, NULL}
+  };
+  status = parsefile("/proc/cpuinfo", archoptions1, 2, 1);
+  status |= parsefile("/proc/cpuinfo", archoptions2, 2, 1);
   if (status || !model || !*model) {
     fprintf( stderr, _("ARM machine type unknown!\n") );
     model = "";
   }
   Arch3 = model;
-  if (strncasecmp(model, "acorn-riscpc", 6) == 0)
+  if (strncasecmp(model, "acorn-", 6) == 0)
     model = "riscpc";
-  if (strcasecmp(model,"chalice-cats") == 0
+  else if (strcasecmp(model, "chalice-cats") == 0
       || strcasecmp(model, "rebel-netwinder") == 0
       || strcasecmp(model, "ebsa285") == 0)
     model = "netwinder";
+  else if (strcasecmp(model, "lart") == 0)
+    model = "lart";
+  else if (strcasecmp(model, "shark") == 0)
+    model = "shark";
+  if (proc && strstr(proc, "StrongARM-110 ")) {
+    char *rev_str = strstr(proc, " rev ");
+    if (rev_str) {
+      int rev_nr = atoi(rev_str + 5);
+      if (rev_nr < 3)
+	strongarm_is_broken = 1;
+    }
+  }
   return model;
 }
 #endif
 
+#if #cpu(mips)
+static char* get_mips_model(void)
+{
+
+  /*
+   * DECstation 5000/20,25,33 is KN02-CA (R3k), should work with r3k-kn02
+   * DECstation 5000/50 is ?? (R4k), should work with r4k-kn02 
+   * DECstation 5000/120,125,133 is KN02 (R3k)
+   * DECstation 5000/150 is KN04 (R4k)
+   * DECstation 5000/240 is ??, should work with r3k-kn02 
+   * DECstation 5000/260 is KN05 (R4k4), should work with r4k-kn04
+   */
+
+  char sys_type[80]; /* 32 bytes is too short for some types of DECstation */
+  char cpu_type[80];
+
+  memset (sys_type, 0, sizeof (sys_type));
+  memset (cpu_type, 0, sizeof (cpu_type));
+  if (parse_cpuinfo("system type", sys_type, sizeof(sys_type))) {
+    /* safe enough since all of them use arc firmware */
+    if (! strncmp(sys_type, "SGI",3)) {
+      return strdup("r4k-ip22");
+    } else if ((! strncmp (sys_type, "Digital DECstation 5000/1xx", 27)) || 
+           (! strncmp (sys_type, "Digital Personal DECstation 5000/xx", 35)) ||
+           (! strncmp (sys_type, "Digital DECstation 5000/2x0", 27))) 
+    {
+      if (parse_cpuinfo("cpu model", cpu_type, sizeof(cpu_type))) {
+        if ((! strncmp(cpu_type, "R4000",5)) ||
+           (! strncmp(cpu_type, "R4400",5)))
+        {
+          return strdup("r4k-kn04");
+        } else {
+          return strdup("r3k-kn02");
+        }
+      }
+    }
+  }
+  return strdup("unknown");
+}
+#endif
+
+#if #cpu(hppa)
+static char* get_hppa_model(void)
+{
+  struct utsname buf;
+
+  if (uname(&buf) < 0)
+    return "";
+  else if (!strcmp(buf.machine, "parisc64"))
+    return "64";
+  else if (!strcmp(buf.machine, "parisc"))
+    return "32";
+  else
+    return "";
+}
+#endif
+
 void get_subarch_name(void)
 {
 #if #cpu(m68k)
@@ -347,6 +460,10 @@ void get_subarch_name(void)
   Arch2 = get_sparc_model();
 #elif #cpu(arm)
   Arch2 = get_arm_model();
+#elif #cpu(mips)
+  Arch2 = get_mips_model();
+#elif #cpu(hppa)
+  Arch2 = get_hppa_model();
 #else
   Arch2 = strdup(""); /* prevent errors */
 #endif
@@ -355,14 +472,14 @@ void get_subarch_name(void)
 void get_kver(void)
 {
   FILE *f;
-  char osrelease[20];
+  char osrelease[64];
   if ((f = fopen("/proc/sys/kernel/osrelease", "r"))) {
-    fscanf(f, "%19s", osrelease);
+    fscanf(f, "%63s", osrelease);
     fclose(f);
     kver = strdup(osrelease);
   } else {
     /* panic! */
-    kver = strdup("2.2.14"); /* default for now */
+    kver = strdup("2.2.19"); /* default for now */
   }
 }
 
@@ -378,6 +495,17 @@ try_to_eject_root_floppy(void)
 }
 #endif
 
+/* try to eject root floppy */
+#if #cpu(powerpc)
+void
+try_to_eject_root_floppy(void)
+{
+ /* Eject that pesky root floppy for PowerMacs */
+  if ((strstr(Arch2, "PowerMac")) && (!strcmp(Arch3, "OldWorld")))
+      eject_floppy("/dev/fd0");
+}
+#endif
+
 #ifdef SCSI_FLOPPY
 /* scan /proc/scsi/scsi for TEAC FC-1 floppy drive
  * and create /dev/sfd[0-7] symlinks if found
@@ -465,8 +593,12 @@ setup_image_names(void)
       else if (!strcmp(Arch2, "CHRP"))
 	 arch_path = "chrp";
       else if (!strcmp(Arch2, "PowerMac"))
-	 arch_path = "powermac";
-      else if (!strcmp(Arch2, "PReP"))
+	  arch_path = "powermac";
+      else if (!strcmp(Arch2, "NewPowerMac")) {
+	  arch_path = "new-powermac";
+	  /* UGLY UGLY UGLY */
+	  bootargs.disksize = "2.88";
+      } else if (!strcmp(Arch2, "PReP"))
 	 arch_path =  "prep";
 
 #elif #cpu(m68k)
@@ -488,18 +620,27 @@ setup_image_names(void)
 #elif #cpu(alpha)
       /* The rescue disk is generic for most Alphas, but MILO is not.
          So we have to fix up here :P */
-      if (strcmp(Arch2, "nautilus") && strcmp(Arch2, "jensen"))
-	arch_path = "";
+      if (   !(!strcmp(Arch2, "nautilus") && !srm_boot)
+	  && !(!strcmp(Arch2, "jensen")))
+	arch_path = NULL;
       /* Furthermore, choose_medium() is really broken, so we have to
          cook up a MILO path here while we're at it */
       sprintf(milo_binary_path, "MILO/%s", Arch2);
+
+#elif #cpu(arm)
+      if (!strcmp(Arch2, "netwinder"))
+	/* UGLY UGLY UGLY */
+	bootargs.disksize = "2.88";
+
 #endif
 
-      strcat(kernel_image_path, arch_path);
-      strcat(kernel_image_path, "/");
+      if (arch_path != NULL) {
+	  strcat(kernel_image_path, arch_path);
+	  strcat(kernel_image_path, "/");
 
-      strcat(drivers_path, arch_path);
-      strcat(drivers_path, "/");
+	  strcat(drivers_path, arch_path);
+	  strcat(drivers_path, "/");
+      }
     }
   }
 
@@ -567,6 +708,7 @@ setup_image_names(void)
   DEBUGMSG("drivers_path is '%s'", drivers_path);
 }
 
+#ifndef _TESTING_
 /* This simply checks that /proc/version exists, which should give us a
  * good indicator that /proc is in fact mounted - BenC
  */
@@ -579,16 +721,22 @@ static void check_proc (void) {
     }
 }
 
-#ifndef _TESTING_
 int main (void) {
     char *term;
     struct serial_struct ser;
 
+    /* SIGCHLD must be DEFAULT and not IGNORE for popen/pclose to work */
+    signal(SIGCHLD, SIG_DFL);
     /* make sure that /proc is mounted */
     check_proc();
     /* Downgrade kernel verbosity */
     klogctl(8, NULL, 4);
 
+    /* ensure sane umask */
+    umask(022);
+
+    INFOMSG("dbootstrap starting; built " BUILDTIME " for " ARCHNAME);
+
     readcmdline();
 
     /* Default disksize, in case disksize= is not set in bootloader config. */
@@ -597,10 +745,13 @@ int main (void) {
     }
 
     if ( bootargs.isverbose && bootargs.isquiet ) {
-      ERRMSG("both verbose and quiet set; turning off quiet");
+      ERRMSG("both verbose and quiet set, going with verbose");
       bootargs.isquiet = 0;
     }
 
+    if (bootargs.isdebug)
+      install_segv_handler();
+
     /* get kernel version */
     get_kver();
 
@@ -616,12 +767,15 @@ int main (void) {
     get_alpha_boot_method();
 #endif
 
+    /* prompt first on install */
+    base_cd=NULL;
+    kernel_cd=NULL;
+    wants_cd=0;
+
     /* setup rescue & drivers disks images filenames
      * (should be done before the first call to stderrToTTY) */
     setup_image_names();
 
-    DEBUGMSG("dbootstrap debugging mode started, $Revision: 1.113 $");
-
     /* Are we on a serial console? */
     if (ioctl(0, TIOCGSERIAL, &ser) < 0)
       /* Not on a serial console */
@@ -629,7 +783,7 @@ int main (void) {
     else {
       /* Serial console on ttySx */
       serialConsole = ser.line;
-      DEBUGMSG("serial console found on line %d", serialConsole);
+      INFOMSG("serial console found on line %d", serialConsole);
     }
     /* Set term type and initialize it. */
     term = getenv("TERM");
@@ -638,54 +792,78 @@ int main (void) {
       setMono();
     }
 #ifdef USE_LANGUAGE_CHOOSER
+    slurp_language_pack ();
+
     setlocale (LC_CTYPE, "");   /* Language chooser relies on this call! */
 #endif
     boxInit();
 
 #ifdef USE_LANGUAGE_CHOOSER
-    if ((lang = boxChooseLanguageVariant (available_languages ())) != NULL)
     {
         char *msgcat = (char *)malloc (PATH_MAX);
+	char *locale = getenv("LC_CTYPE");
+	const struct language_definition *langs = available_languages ();
 
-        snprintf (msgcat, PATH_MAX, "/etc/messages.%s", lang->msgcat);
-        msgcat[PATH_MAX - 1] = '\0';
-
-        DEBUGMSG ("loading message catalog %s", msgcat);
-
-        if (LOAD_TRMFILE (msgcat) == 0)         /* Failed to load localized strings? */
-        {
-            if (LOAD_TRMFILE (TRMBACKUP) == 0)  /* Failed to load English strings as well? */
+	if (locale && strstr(locale, "utf-8")) {
+	  /* UTF-8 mode, choose a language */
+	  do {
+	    lang = boxChooseLanguageVariant (langs);
+	  } while (lang == NULL);
+	  snprintf (msgcat, PATH_MAX, "/etc/messages.%s", lang->msgcat);
+	  msgcat[PATH_MAX - 1] = '\0';
+	  DEBUGMSG ("loading message catalog %s", msgcat);
+	  
+	  if (LOAD_TRMFILE (msgcat) == 0)         /* Failed to load localized strings? */
+	  {
+	    if (LOAD_TRMFILE (TRMBACKUP) == 0)  /* Failed to load English strings as well? */
             {
-                problemBoxEn ("An error occured while loading application messages.", "Problem");
+	      problemBoxEn ("An error occured while loading application messages.", "Problem");
+	      
+	      reboot (RB_AUTOBOOT);
+	      
+	      /* when not root and debugging */
+	      boxFinished();
+	      exit(1); 
+	    }
+	    else
+	    {
+	      char message[255];
 
-                reboot (RB_AUTOBOOT);
+	      snprintf (message, sizeof (message), "An error occured while loading localized application messages from '%s'. English messages will be used instead.", msgcat);
+	      message[254] = '\0'; /* Just in case */
+	      
+	      problemBoxEn (message, "Problem");
+	    }
+	  }
+	} else {
+	  /* Try to force the language to English */
+	  while (langs->name != NULL && strcmp(langs->name, "English"))
+	    langs++;
+	  if (langs->name == NULL) {
+	    problemBoxEn ("UTF-8 not available and no default language present -- can't continue", "Problem");
+	    reboot (RB_AUTOBOOT);
+	  }
+	  lang = langs->list->items[0].p;
 
-                /* when not root and debugging */
-                boxFinished();
-                exit(1); 
-            }
-            else
-            {
-                char message[255];
+	  snprintf (msgcat, PATH_MAX, "/etc/messages.%s", lang->msgcat);
+	  msgcat[PATH_MAX - 1] = '\0';
+	  DEBUGMSG ("i18n mode, loading message catalog %s", msgcat);
 
-                snprintf (message, sizeof (message), "An error occured while loading localized application messages from '%s'. English messages will be used instead.", msgcat);
-                message[254] = '\0'; /* Just in case */
-
-                problemBoxEn (message, "Problem");
-            }
+	  /* The English messages might be /etc/messages.en, or
+	     /etc/messages.trm in a single locale system */
+	  if (LOAD_TRMFILE (msgcat) == 0
+	      && LOAD_TRMFILE ("/etc/messages.trm") == 0)
+	  {
+	    problemBoxEn ("An error occurred while loading application messages", "Problem");
+	    reboot(RB_AUTOBOOT);
+	    /* when not root and debugging */
+	    boxFinished();
+	    exit(1); 
+	  }
 	}
+
         free (msgcat);
     }
-    else
-    {
-        problemBoxEn ("You failed to choose a language.  Rebooting...", "Problem");
-
-        reboot (RB_AUTOBOOT);
-
-        /* when not root and debugging */
-        boxFinished();
-        exit (1);
-    }
 #else
     /* We can use problemBox only AFTER boxInit */
     /* So it's important that no (NO!!!) function before this call (LOAD_TRMFILE) uses gettext function */
@@ -716,11 +894,9 @@ int main (void) {
     if (strcmp(Arch2, "VME") == 0)
       klogctl(6, NULL, 0);
 #elif #cpu (powerpc)
-    if ((strncasecmp(Arch2, "Power", 5) == 0)
-     || (strncasecmp(Arch2, "iMac",  4) == 0)) Arch2 = "PowerMac";
-    else
-    if ( (strcmp(Arch2,"CHRP")) && (strcmp(Arch2,"PReP"))
-     && (strcasecmp(Arch2,"apus"))) {
+    if ( !strstr(Arch2,"PowerMac") &&
+	 strcmp(Arch2,"CHRP") && strcmp(Arch2,"PReP")
+	 && strcasecmp(Arch2,"apus")) {
       problemBox(_("Your PowerPC architecture is not supported yet."), _("Problem"));
       reboot(RB_AUTOBOOT);
     }
@@ -730,7 +906,7 @@ int main (void) {
     find_scsi_floppy();
 #endif
 
-#if #cpu(sparc)
+#if #cpu(sparc) || #cpu(powerpc)
     try_to_eject_root_floppy();
 #endif
 
@@ -742,7 +918,7 @@ int main (void) {
 
     InstallationRootDevice=block_device("/");
     if (!InstallationRootDevice) {
-      /* net installation with no /dev/root device on nfsroot */
+      /* something's borked, cannot determine where / comes from */
       /* leave time to see any error message */
       sleep(8);
       problemBox(_("I had trouble checking the choice of root device. I don't know what to use as the root fs."),_("Problem"));
@@ -760,6 +936,50 @@ int main (void) {
 #else
     release_notes (NULL);
 #endif
+
+#if #cpu(arm)
+    if (strongarm_is_broken)
+      wideMessageBox(_(
+"This machine seems to be using an old StrongARM CPU.  SA-110\n"
+"processors prior to revision S suffered from bugs that will prevent\n"
+"Linux from running reliably.\n\n"
+"You can go ahead and install anyway, but you may experience problems\n"
+"later.\n\n"
+"For more information, refer to http://www.armlinux.org/strongarm.html\n"), 
+		     _("Warning"));
+#endif
+
+    /* See how much memory is fitted.  If it's less than the
+       recommended amount, say so. */
+    {
+      char *smem = NULL;
+      int status;
+      struct Arg args[] = { 
+	{ "MemTotal", 0, &smem }, 
+	{ NULL, 0, NULL } 
+      };
+      status = parsefile("/proc/meminfo", args, 2, 1);
+      if (status == 0 && smem != NULL) {
+	unsigned long mem = atol(smem), min_mem;
+	mem /= 1024;
+	/* This information comes from documentation/default.ent */
+#if defined __i386__
+	min_mem = 12;
+#elif defined __arm__ || defined __s390__ || defined __alpha__ || defined __mips__ || defined __powerpc__
+	min_mem = 16;
+#else
+	min_mem = 5;
+#endif
+	if (mem < min_mem) {
+	  snprintf(prtbuf, PRTBUFSIZE - 1, _(
+"Your computer seems to have only %ldMB of RAM.  For this\n"
+"version of Debian, a minimum of %ldMB is recommended.\n"), 
+		   mem, min_mem);
+	  wideMessageBox(prtbuf, _("Warning: Low Memory"));
+	}
+      }
+    }
+
     is_root_a_floppy ();
 
     main_menu ();
