@@ -1,4 +1,4 @@
-
+ 
 #include <stdio.h>
 #include <assert.h>
 #include <sys/sysmacros.h>
@@ -30,13 +30,13 @@ int init_linux(void);
 int mount_any(void);
 int unmount_any(void);
 int extract_kernel_and_modules(void); 
-int extract_base(void);
+int debootstrap_extract_base(void);
 int configure_drivers(void);
-int configure_base(void);
 int configure_network(void);
 int make_bootable(void);
 int make_boot_floppy(void);
 int reboot_system(void);
+int halt_system(void);
 int chroot_start(void);
 int view_partitions(void);
 int interactive_shell(void);
@@ -45,6 +45,8 @@ int do_eject(void);
 int load_modules_after_boot(void);
 int load_modules_after_boot_ac(void);
 int edit_append_line();
+int add_modules_from_floppy(void);
+int attach_disk(void);
 
 #else //!_TESTING_
 
@@ -84,6 +86,7 @@ require_root (void)
 	return 0;
 }
 
+
 #ifdef DO_EJECT
 void
 eject_floppy (const char *device)
@@ -92,8 +95,12 @@ eject_floppy (const char *device)
 	execlog (prtbuf, LOG_INFO);
 }
 #endif
+#endif //!_TESTING_
 
-#endif //_TESTING_
+int main_menu_extract_base ()
+{
+	return failedBaseInstall = debootstrap_extract_base ();
+}
 
 int
 main_menu (void)
@@ -101,6 +108,7 @@ main_menu (void)
 #ifndef _TESTING_
 	char *found_root = NULL;
 #endif
+	int offerred_keyboard = 0;
 	struct stat statbuf;
 	int (*next_action) (void);
 	int (*alternate_action) (void);
@@ -124,7 +132,6 @@ main_menu (void)
 #define OPT_INSTALL_OS		1014
 #define OPT_INSTALL_BASE	1015
 #define OPT_CONFIGURE_MODULES	1016
-#define OPT_CONFIGURE_BASE	1017
 #define OPT_CONFIGURE_NET	1018
 #define OPT_DISK_BOOT		1019
 #define OPT_FLOPPY_BOOT		1020
@@ -138,11 +145,19 @@ main_menu (void)
 #define OPT_CONFIGURE_HOSTNAME	1028
 #define OPT_LOAD_MODULES_FROM_A_FD_AFTER_BOOT 1029
 #define OPT_KERNEL_APPEND	1030
+#define OPT_PROBLEM_REPORT	1031
+#define OPT_FOREIGN_MODULES	1032
+#define OPT_ATTACH_DISK   	1033
 #define NITEMS	31
 	int data, ilist[NITEMS], items, i, rs;
 	struct d_choices choices[NITEMS];
 	struct { char *string; int key; } entries[] = {
+#if !(#cpu(s390))
 		{_("Configure the Keyboard"),				OPT_CONFIGURE_KEY},
+#endif
+#if (#cpu(s390))
+		{_("Attach a Disk"),					OPT_ATTACH_DISK},
+#endif
 /* since this process uses lilo, we only use it with i386 */
 #if (#cpu(i386))
 		{_("Preload modules from a floppy"),	OPT_LOAD_MODULES_FROM_A_FD_AFTER_BOOT},
@@ -154,26 +169,31 @@ main_menu (void)
 		{_("Initialize a Linux Partition"),			OPT_INITIALIZE_LINUX},
 		{_("Mount a Previously-Initialized Partition"),		OPT_MOUNT_LINUX},
 		{_("Unmount a Partition"),				OPT_UMOUNT_LINUX},
-		{_("Install Operating System Kernel and Modules"),	OPT_INSTALL_OS},
+		{_("Install Kernel and Driver Modules"),		OPT_INSTALL_OS},
 		{_("Configure Device Driver Modules"),			OPT_CONFIGURE_MODULES},
-#if !(#cpu(alpha) || #cpu(m68k))
+#ifdef PCMCIA
 		{_("Configure PCMCIA Support"),				OPT_CONFIGURE_PCMCIA},
+#endif
+#if !(#cpu(s390))
+		{_("Install Foreign Modules"),				OPT_FOREIGN_MODULES},
 #endif
 		{_("Configure the Hostname"),				OPT_CONFIGURE_HOSTNAME},
 		{_("Configure the Network"),				OPT_CONFIGURE_NET},
 		{_("Install the Base System"),				OPT_INSTALL_BASE},
-		{_("Configure the Base System"),			OPT_CONFIGURE_BASE},
-#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc))
+#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc)) || #cpu(powerpc)
 		{_("Edit Kernel Boot Parameters"),			OPT_KERNEL_APPEND},
 #endif
-		{_("Make Linux Bootable Directly From Hard Disk"),	OPT_DISK_BOOT},
+		{_("Make System Bootable"),				OPT_DISK_BOOT},
+#if !(#cpu(hppa) || #cpu(s390) || #cpu(mips) || #cpu(mipsel) || #cpu(ia64) || #cpu(arm))
 		{_("Make a Boot Floppy"),				OPT_FLOPPY_BOOT},
+#endif
 #if 0 /* not working yet */
 		{_("Start New System"),					OPT_START_NEW_SYSTEM},
 #endif
 		{_("Reboot the System"),				OPT_REBOOT},
 		{_("View the Partition Table"),				OPT_VIEW_PARTITIONS},
 		{_("Execute a Shell"),					OPT_EXECUTE_SHELL},
+		{_("Report a Problem"),					OPT_PROBLEM_REPORT},
 #ifdef DO_EJECT
 		{_("Eject Floppy Disks"),				OPT_EJECT_FLOPPY},
 #endif
@@ -193,21 +213,24 @@ main_menu (void)
 		{_("Unmount a Partition"),				OPT_UMOUNT_LINUX},
 		{_("Install Operating System Modules"),			OPT_INSTALL_OS},
 		{_("Configure Device Driver Modules"),			OPT_CONFIGURE_MODULES},
+		{_("Install Foreign Modules"),				OPT_FOREIGN_MODULES},
 		{_("Configure the Hostname"),				OPT_CONFIGURE_HOSTNAME},
 		{_("Configure the Network"),				OPT_CONFIGURE_NET},
 		{_("Install the Base System"),				OPT_INSTALL_BASE},
-		{_("Configure the Base System"),			OPT_CONFIGURE_BASE},
 		{_("Reboot the System"),				OPT_REBOOT},
 		{_("View the Partition Table"),				OPT_VIEW_PARTITIONS},
 		{_("Execute a Shell"),					OPT_EXECUTE_SHELL},
 		{_("Restart Installation System"),			OPT_RESTART_DBOOTSTRAP},
+		{_("Report a Problem"),					OPT_PROBLEM_REPORT},
 		{NULL,0}
 	};
 #endif
 
 	for (;;) {
 		pleaseWaitBox (
-			_("The installation program is determining the current state of your system and the next installation step that should be performed."));
+			_("The installation program is determining the current state "
+			  "of your system and the next installation step that should "
+			  "be performed."));
 		memset (choices, 0, NITEMS * sizeof (struct d_choices));
 
 #ifndef _TESTING_
@@ -216,12 +239,14 @@ main_menu (void)
 	}
 
 	fdisk_reread();
+#if !(#cpu(s390))
 	if (fdisk_disks == NULL) {
 	    /* no swap on NFS for diskless workstation */
 	    noSwap = 1;
 	    /* no boot loader on local disk */
 	    notInstalledLILO = 0;
 	}
+#endif
 
 	if (found_root) {
 	    Root = fdisk_find_partition_by_name (found_root);
@@ -266,12 +291,17 @@ main_menu (void)
 	if ((! NAME_ISREG (KEYBD_SETTINGS_FILE, &statbuf))
 	    && NAME_ISEXE ("/sbin/loadkmap", &statbuf) 
 	    && (serialConsole < 0)
+	    && (!offerred_keyboard)
 #if #cpu (m68k)
 	    && (strcmp(Arch2, "VME") != 0)
+#endif
+#if #cpu (s390)
+	    && 0                /* no keyboard settings for s390 needed*/ 
 #endif
 		) {
 
-#ifdef USE_LANGUAGE_CHOOSER
+		offerred_keyboard = 1;
+#if defined(USE_LANGUAGE_CHOOSER) && defined(USE_LANGUAGE_KEYMAPS)
 		/* assert (lang != NULL); */
 		if (lang->keymap != NULL && lang->keymap[0] != '\0')
 		{
@@ -288,10 +318,18 @@ main_menu (void)
 		if (bootargs.kbd) {
 			configure_keyboard (bootargs.kbd);
 			goto done_keyboard;
+#ifdef KBD
+                        
+		} else {
+                        /* change to keyboard defined in config (only if bootargs.kbd != NULL) */
+			configure_keyboard (KBD);
+			goto done_keyboard;
+#endif
 		}
 
+
 		next_action = configure_keyboard_m;
-		bufT = _("Your keyboard has not yet been configured. Please select \"Next\" from the menu to configure the keyboard.");
+		bufT = _("You must indicate what sort of keyboard you have so that keys operate as expected.  Select \"Next\" from the menu to configure your keyboard.");
 		bufN = _("Configure the Keyboard");
 #if ( #cpu(i386) )
 		bufA = _("Preload essential modules from a floppy");
@@ -328,13 +366,16 @@ main_menu (void)
 	}
 	else if ((swapon_partitions == NULL)
 		 && (fdisk_partitions_by_type[FSTYPE_SWAP] == NULL)
+#if ( #cpu(s390) )
+		 && (fdisk_disks != NULL)
+#endif
 		 && (noSwap == 0)) {
 		/*
 		 * No "Linux swap" partitions have been created.
 		 */
 		next_action      = partition_disk;
 		alternate_action = no_swap;
-		bufT = _("There are no \"Linux swap\" partitions present on the system. A swap partition is used to provide virtual memory for Linux. Please select \"Next\" from the menu to partition your hard disk. Use the partitioning program to add \"Linux native\" and \"Linux swap\" partitions to your disks. If you don't want a swap partition, select \"Alternative\".");
+		bufT = _("A swap partition is strongly recommended to provide virtual memory for your system, yet no swap partitions were detected.  Selecting \"Next\" will start the partitioning program.  Use that to create \"Linux native\" and \"Linux swap\" partitions on your disks.  If you don't want a swap partition, select \"Alternative\".");
 		bufN = _("Partition a Hard Disk");
 		bufA = _("Do Without a Swap Partition");
 #if ( #cpu(i386) )
@@ -342,8 +383,23 @@ main_menu (void)
 			alternate1_action = load_modules_after_boot;
 			bufA1 = _("Preload essential modules from a floppy");
 		}
+#elif ( #cpu(s390) )
+		alternate1_action = attach_disk;
+		bufA1 = _("Attach a Disk");
 #endif
 	}
+#if ( #cpu(s390) ) &&  !defined(NFSROOT)
+	else if ((! Root) && (fdisk_disks == NULL) ) {
+		/*
+		 * No disks found:
+		 *
+		 * Attach disk dynamically.
+		 */
+		next_action = attach_disk;
+		bufN = _("Attach a Disk");
+		bufT = _("No hard disk drives were detected.  You need to attach disks dynamically.  Select \"Next\" to attach a disk.");
+	}
+#endif
 	else if ((! Root) && (fdisk_disks == NULL) && is_network_up(0)) {
 		 /*
 		  * No disks found, but ethernet is up
@@ -356,9 +412,13 @@ main_menu (void)
 #if ( #cpu(i386) )
 		alternate_action = load_modules_after_boot;
 		bufA = _("Preload essential modules from a floppy");
-		bufT = _("No hard disk drives could be found but the network is configured.  Please select \"Next\" to mount the root filesystem via NFS. Please select \"Alternate\" if you need to load essential modules from a floppy to make the hard disk drives visible.");
+		bufT = _("No hard disk drives could be found and the network is configured.  Please select \"Next\" to mount the root filesystem via NFS.  Select \"Alternate\" instead if you need to load essential modules from a floppy to make the hard disk drives visible.");
+#elif ( #cpu(s390) )
+		alternate_action = attach_disk;
+		bufA = _("Attach a Disk");
+		bufT = _("No hard disk drives could be found and the network is configured.  Please select \"Next\" to mount the root filesystem via NFS.  Select \"Alternate\" instead if you want to attach a disk.");
 #else
-		bufT = _("No hard disk drives could be found but the network is configured.  Please select \"Next\" to mount the root filesystem via NFS.");
+		bufT = _("No hard disk drives could be found and the network is configured.  Please select \"Next\" to mount the root filesystem via NFS.");
 #endif
 	}
 #if ( #cpu(i386) )
@@ -373,7 +433,7 @@ main_menu (void)
 		alternate_action = configure_network;
 		bufN = _("Preload essential modules from a floppy");
 		bufA = _("Configure the Network");
-		bufT = _("No hard disk drives could be found. You either need to load special drivers from a floppy or plan to install the system on a diskless workstation, whose disks are mounted via NFS.  Please select \"Next\" to load essential modules from a floppy or \"Alternate\" to configure the network. ");
+		bufT = _("No hard disk drives were detected.  You either need to load special drivers from a floppy, or you plan to install the system on a diskless workstation which uses NFS in place of local disk.  Select \"Next\" to load essential modules from a floppy, or \"Alternate\" to configure the network.");
 	}
 #endif
 
@@ -384,7 +444,7 @@ main_menu (void)
 		  * Need to configure the network before mounting "/target".
 		  */
 		next_action = configure_network;
-		bufT = _("No hard disk drives could be found.  You're installing the system on a diskless workstation, therefore you need to configure the network before mounting your Linux root filesystem from a NFS server.  Please select \"Next\" to configure the network."); 
+		bufT = _("No hard disk drives were detected.  Since you must be installing a diskless workstation, you must configure the network before mounting your Linux root filesystem from a NFS server.  Select \"Next\" to configure the network."); 
 		bufN = _("Configure the Network");
 #if ( #cpu(i386) )
 		if ( ! load_modules_after_boot_ac() ) {
@@ -398,8 +458,12 @@ main_menu (void)
 		 * No "Linux native" partitions have been created.
 		 */
 		next_action = partition_disk;
-		bufT = _("There are no \"Linux native\" partitions present on the system. You must create at least one \"Linux native\" partition to hold the root filesystem, and you may create additional ones. Please select \"Next\" from the menu to partition your hard disk. Use the partitioning program to add \"Linux native\" partitions to your disks.");
+		bufT = _("No Linux partitions were detected.  At least one Linux partition is required to hold the root filesystem.  Selecting \"Next\" will start the partitioning program.  Use that to create \"Linux native\" partitions on your local disk.");
 		bufN = _("Partition a Hard Disk");
+#if ( #cpu(s390) )
+		alternate_action = attach_disk;
+		bufA = _("Attach a Disk");
+#endif
 	}
 	else if ((swapon_partitions == NULL) && (noSwap == 0)) {
 		/*
@@ -410,7 +474,7 @@ main_menu (void)
 		alternate_action  = activate_swap;
 		alternate1_action = no_swap;
 		previous_action   = partition_disk;
-		bufT = _("There are \"Linux native\" and \"Linux swap\" partitions present on your system. The next step would be to initialize and activate a swap partition to provide virtual memory to your system. If you have not finished partitioning your disks or wish to change the partitions, please select \"Previous\" from the menu to partition them. If you are satisfied with your partitions, please select \"Next\" from the menu to initialize and activate your swap partition, or \"Alternate\" to activate a previously-initialized swap partition. If you absolutely insist on doing without a swap partition, select \"Alternate1\".");
+		bufT = _("You have a swap partition, but it needs to be initialized and activated.  Select \"Next\" to put this swap partition to use, providing virtual memory for your system.  Select \"Alternate\" to activate a swap partition which has already been initialized for swap.\n\nIf you have not finished partitioning your disks, select \"Previous\".  If you do not want to use your swap partition, select \"Alternate1\".");
 		bufN = _("Initialize and Activate a Swap Partition");
 		bufA = _("Activate a Previously-Initialized Swap Partition");
 		bufA1 = _("Do Without a Swap Partition");
@@ -422,9 +486,13 @@ main_menu (void)
 		 */
 		next_action      = init_linux;
 		alternate_action = mount_any;
-		bufT = _("There are \"Linux native\" partitions present on your system but none are mounted. You must mount a root filesystem, and you may mount other filesystems, before installing the system. If you have just created the partitions, you must initialize them before you can mount them. Please select \"Next\" to initialize and mount a Linux partition, or \"Alternate\" to mount a previously-initialized partition.");
+		bufT = _("There are Linux partitions present on your system, but none are mounted. You must mount a partition as the root filesystem before continuing.  If you have just created the partitions, select \"Next\" to initialize (or \"format\") a partition.  If you have partitions which have already been initialized, use \"Alternate\" to mount them without initialization.");
 		bufN = _("Initialize a Linux Partition");
 		bufA = _("Mount a Previously-Initialized Partition");
+#if ( #cpu(s390) )
+		alternate1_action = attach_disk;
+		bufA1 = _("Attach a Disk");
+#endif
 	}
 	else if (! Root && fdisk_disks == NULL) {
 		/*
@@ -432,19 +500,21 @@ main_menu (void)
 		 * is a diskless NFS-root install.
 		 */
 		next_action = mount_any;
-		bufT = _("You must mount a root filesystem, and you may mount other filesystems, before installing the system. To do a diskless install you must mount your Linux root filesystem from a NFS server. Please select \"Next\" to mount a NFS partition.");
+		bufT = _("You must mount a root filesystem before continuing.  Since you have no local disk, you probably want to mount the root filesystem over the network from an NFS server.  Selecting \"Next\" will let you do that.");
 		bufN = _("Mount a Previously-Initialized Partition");
 	}
 	else if (((! NAME_ISREG ("/target/vmlinuz", &statbuf)) /* Assumes that NAME_ISREG uses stat() and thus follows the symlink. #### */
-		  && (! NAME_ISREG ("/target/vmlinux", &statbuf)))
+		  && (! NAME_ISREG ("/target/vmlinux", &statbuf))
+		  && (! NAME_ISREG ("/target/boot/vmlinuz", &statbuf)) /* The link is moved to /boot on some platforms, eg. hppa */
+		  && (! NAME_ISREG ("/target/boot/vmlinux", &statbuf))) /* powerpc needs this */
 		 || (! NAME_ISREG ("/target/usr/share/modconf/descr.gz", &statbuf))) {
 		/*
 		 * The operating system kernel and the modules have
 		 * not been installed.
 		 */
 		next_action               = extract_kernel_and_modules;
-		bufT = _("You've mounted your root filesystem. You may initialize and mount additional filesystems, or you may go on to install the operating system kernel and the modules.");
-		bufN = _("Install Operating System Kernel and Modules");
+		bufT = _("You've mounted your root filesystem.  You may initialize and mount additional filesystems, or you may go on to install the operating system kernel and the modules.");
+		bufN = _("Install Kernel and Driver Modules");
 		if (fdisk_disks != NULL) {
 			alternate_action  = init_linux;
 			alternate1_action = mount_any;
@@ -462,27 +532,27 @@ main_menu (void)
 		 */
 		next_action              = configure_drivers;
 		bufN = _("Configure Device Driver Modules");
-#if !(#cpu(alpha) || #cpu(m68k))
+#ifdef PCMCIA
 		if (! NAME_ISREG ("/target/etc/pcmcia.conf", &statbuf)) {
-			bufT = _("You've installed the operating system kernel and the modules. Please select \"Next\" to configure the device driver modules, or \"Alternate\" if you have to configure any PCMCIA devices."); 
+			bufT = _("You've installed the operating system kernel and the modules.  Please select \"Next\" to configure the device driver modules, \"Alternate\" if you have to configure any PCMCIA devices or \"Alternate1\" to install 3rd party modules."); 
 			alternate_action = configure_pcmcia;
 			bufA = _("Configure PCMCIA Support");
-#if (#cpu(i386) || #cpu(sparc))
-		if (!strlen(append_opts)) {
-			alternate1_action = edit_append_line;
-			bufA1 = _("Edit Kernel Boot Parameters");
-		}
-#endif
+			alternate1_action = add_modules_from_floppy;
+			bufA1 = _("Install Foreign Modules");
 		} else
 #endif
 		{
-#if (#cpu(alpha) || #cpu(m68k))
-		if (!strlen(append_opts)) {
-			alternate_action = edit_append_line;
-			bufA = _("Edit Kernel Boot Parameters");
-		}
+#if !(#cpu(s390))
+			alternate_action = add_modules_from_floppy;
+			bufA = _("Install Foreign Modules");
 #endif
-			bufT = _("You've installed the operating system kernel and the modules. Please select \"Next\" to configure the device driver modules.");
+#if (#cpu(alpha) || #cpu(m68k))
+			if (!strlen(append_opts)) {
+	    		    alternate1_action = edit_append_line;
+	    		    bufA1 = _("Edit Kernel Boot Parameters");
+			}
+#endif
+			bufT = _("You've installed the operating system kernel and the modules.  Please select \"Next\" to configure the device driver modules or \"Alternate\" to install 3rd party modules.");
 		}
 	}
 	else if (! NAME_ISREG(target_path(NC_HOSTNAME_FILE),&statbuf) && !is_network_up(1)) {
@@ -494,11 +564,13 @@ main_menu (void)
 		 */
 		next_action	= configure_hostname;
 		bufN = _("Configure the hostname");
-		bufT = _("The hostname is the name of the system. You need it even if you aren't setting up this system on a network.");
+		alternate_action = configure_drivers;
+		bufA = _("Configure Device Driver Modules");
+		bufT = _("No network devices are available, therefore, you must set your hostname rather than fully configuring the network.\n\nIf you do have a network device, it is not currently being detected, so you should select \"Alternate\" to attempt to load a kernel driver for it.");
 #if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc))
 		if (!strlen(append_opts)) {
-			alternate_action = edit_append_line;
-			bufA = _("Edit Kernel Boot Parameters");
+			alternate1_action = edit_append_line;
+			bufA1 = _("Edit Kernel Boot Parameters");
 		}
 #endif
 	}
@@ -508,25 +580,25 @@ main_menu (void)
 		 */
 		next_action      = configure_network;
 		if (is_network_up(0))
-		    bufT = _("Your network is active, but it has not been configured for the system to remember. If your system is configuring itself via hardware (i.e. your system can setup it's network before Linux boots through DHCP or BOOTP), you should still setup the configuration so that the network can be brought up and down after booting. If you configured the network yourself, then you also need to perform this step. Please select \"Next\" to configure the network");
+		    bufT = _("Your network is active, but it has not been configured for the system to remember.  If your system is configuring itself via hardware (i.e., your system can setup its network before Linux boots through DHCP or BOOTP), you should still setup the configuration so that the network can be brought up and down after booting.  If you configured the network yourself, then you also need to perform this step.");
 		else if (is_network_configured())
-		    bufT = _("Your network is configured, but not activated. This may be caused by a broken previous installation attempt, or because you are installing over another installation. It is ok to skip this step if you do not need to use the network for the rest of the installation, since your network will become active after reboot (assuming it is setup properly). Please select \"Next\" to reconfigure the network");
+		    bufT = _("Your network is configured, but not activated.  This may be caused by a broken previous installation attempt, or because you are installing over another installation.  It is ok to skip this step if you do not need to use the network for the rest of the installation, since your network will become active after reboot (assuming it is setup properly).");
 		else
-		    bufT = _("You've configured the device drivers but not the network.  Please select \"Next\" to configure the network.");
+		    bufT = _("You have a network device, but it is not yet configured.  Please select \"Next\" to configure the network.");
 		bufN = _("Configure the Network");
 		if (! NAME_ISREG ("/target/etc/debian_version", &statbuf)
 		    || ! NAME_ISREG ("/target/sbin/init", &statbuf)
 		    || failedBaseInstall) {
-			alternate_action = extract_base;
+			alternate_action = main_menu_extract_base;
 			bufA = _("Install the Base System");
-#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc))
+#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc)) || #cpu(powerpc)
 			if (!strlen(append_opts)) {
 				alternate1_action = edit_append_line;
 				bufA1 = _("Edit Kernel Boot Parameters");
 			}
 #endif
 		}
-#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc))
+#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc)) || #cpu(powerpc)
 		else {
 			if (!strlen(append_opts)) {
 				alternate_action = edit_append_line;
@@ -541,28 +613,20 @@ main_menu (void)
 		/*
 		 * The base system has not been installed.
 		 */
-		next_action = extract_base;
+		next_action = main_menu_extract_base;
 		if (failedBaseInstall) {
 			bufT = _("The previous attempt to install the base system failed. Please select \"Next\" to retry installation of the base system.");
 		} else {
-			bufT = _("You've configured the device driver modules. Please select \"Next\" to install the base system.");
+			bufT = _("The next step is to install essential software for your new Debian GNU/Linux system.  This is called the base system.  It will be installed to the disks you have already mounted.");
 		}
 		bufN = _("Install the Base System");
-#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc))
+#if (#cpu(alpha) || #cpu(m68k) || #cpu(i386) || #cpu(sparc)) || #cpu(powerpc)
 		if (!strlen(append_opts)) {
 			alternate_action = edit_append_line;
 			bufA = _("Edit Kernel Boot Parameters");
 		}
 #endif
 	}
-	else if (NAME_ISREG ("/target/sbin/unconfigured.sh", &statbuf)) {
-		/*
-		 * The base system has not been configured.
-		 */
-		next_action = configure_base;
-		bufT = _("You've installed the base system and the operating system kernel. Please select \"Next\" to configure the base system.");
-		bufN = _("Configure the Base System");
-	}
 	else if (notCreatedBootFloppy && notInstalledLILO && ! is_nfs_partition ("/target")
 #if #cpu(m68k)
 		 && (strcmp(Arch2, "Mac") != 0)
@@ -572,14 +636,22 @@ main_menu (void)
 		 * Reboot, chroot, or create a boot floppy.
 		 */
 		next_action               = make_bootable;
+#if !(#cpu(s390) || #cpu(hppa) || #cpu(ia64) || #cpu(mips) || #cpu(mipsel))
 		alternate_action          = make_boot_floppy;
+#endif
 		if (initchroot_cap())
 			alternate1_action = chroot_start;
 		else
+#if !(#cpu(s390))
 			alternate1_action = reboot_system;
-		bufT = _("You will either boot Linux from a floppy disk, or you will have it boot from the hard disk when your system is turned on.  Please select \"Next\" to make Linux bootable the hard disk, select \"Alternate\" to make a boot floppy, or select \"Alternate1\" to reboot the system.");
-		bufN = _("Make Linux Bootable Directly From Hard Disk");
+#else
+			alternate1_action = halt_system;
+#endif
+		bufT = _("Select \"Next\" to configure your system to boot into Debian when powered on.  The \"Alternate\" step will make a custom floppy for booting; this is a good option if you don't want to change how your system currently boots up (for instance, you have another operating system installed).  Select \"Alternate1\" to simply reboot the system, which is appropriate if you have other means of booting, or have configured booting on your own.");
+		bufN = _("Make System Bootable");
+#if !(#cpu(s390) || #cpu(hppa) || #cpu(ia64) || #cpu(mips) || #cpu(mipsel))
 		bufA = _("Make a Boot Floppy");
+#endif
 		if (initchroot_cap())
 			bufA1 = _("Start New System");
 		else
@@ -588,6 +660,9 @@ main_menu (void)
 	else if (notCreatedBootFloppy
 #if #cpu(m68k)
 		 && (strcmp(Arch2, "Mac") != 0)
+#endif
+#if #cpu (s390) || #cpu(hppa) || #cpu(ia64) || #cpu(mips) || #cpu(mipsel)
+	    && 0                /* no boot floppy for s390, hppa, ia64 or mips(el) */ 
 #endif
 		) {
 		/*
@@ -597,8 +672,12 @@ main_menu (void)
 		if (initchroot_cap())
 			alternate_action = chroot_start;
 		else
+#if !(#cpu(s390))
 			alternate_action = reboot_system;
-		bufT = _("You've built your system. You should build a boot floppy that will start the system just in case it doesn't start directly from the hard disk.  Please select \"Next\" to build the boot floppy.");
+#else
+			alternate_action = halt_system;
+#endif
+		bufT = _("Debian is now installed (although not yet fully configured).\n\nYou should build a custom boot floppy, in case Debian GNU/Linux doesn't boot  directly from the hard disk.  Please select \"Next\" to build the custom boot floppy.");
 		bufN = _("Make a Boot Floppy");
 		if (initchroot_cap())
 			bufA = _("Start New System");
@@ -611,15 +690,19 @@ main_menu (void)
 		 */
 		next_action      = chroot_start;
 		alternate_action = reboot_system;
-		bufT = _("You've built your system and boot floppy. The moment of truth approaches! Please select \"Next\" to enter your new system, and continue configurations steps");
+		bufT = _("It's time to start your new Debian GNU/Linux system.  Select \"Next\" to bootstrap your machine without reboot and complete the remaining configuration.");
 		bufN = _("Start New System");
 		bufA = _("Reboot the System");
 	} else {
 		/*
 		 * Nothing left to do but reboot the system.
 		 */
+#if !(#cpu(s390))
 		next_action = reboot_system;
-		bufT = _("You've built your system and boot floppy. The moment of truth approaches!  Please select \"Next\" to reboot the system.");
+#else
+		next_action = halt_system;
+#endif
+		bufT = _("It's time to start your new Debian GNU/Linux system.  Select \"Next\" to reboot your machine and complete the remaining configuration.  If your system has troubles booting, you may have to fall back to your custom boot disk, or the rescue disk -- see the documentation.");
 		bufN = _("Reboot the System");
 	}
 
@@ -696,16 +779,12 @@ main_menu (void)
 	      } else { require_root(); } break;
 	    case OPT_INSTALL_BASE:
 	      if (Root) {
-		failedBaseInstall = extract_base();
+		main_menu_extract_base();
 	      } else { require_root(); } break;
 	    case OPT_CONFIGURE_MODULES:
 	      if (Root) {
 		configure_drivers();
 	      } else { require_root(); } break;
-	    case OPT_CONFIGURE_BASE:
-	      if (Root) {
-		configure_base(); 
-	      } else { require_root(); } break;
 	    case OPT_CONFIGURE_NET:    configure_network(); break;
 	    case OPT_DISK_BOOT:
 	      if (Root) {
@@ -716,17 +795,44 @@ main_menu (void)
 		make_boot_floppy();
 	      } else { require_root(); } break;
 	    case OPT_START_NEW_SYSTEM: chroot_start(); break;
-	    case OPT_REBOOT:           reboot_system(); break;
+	    case OPT_REBOOT:
+#if !(#cpu(s390))
+		reboot_system(); break;
+#else
+		halt_system(); break;
+#endif
 	    case OPT_VIEW_PARTITIONS:  view_partitions(); break;
 	    case OPT_EXECUTE_SHELL:    interactive_shell(); break;
+#ifdef PCMCIA
 	    case OPT_CONFIGURE_PCMCIA:
 	      if (Root) {
 		configure_pcmcia();
 	    } else { require_root(); } break;
+#endif 
 #ifdef DO_EJECT
 	    case OPT_EJECT_FLOPPY: do_eject(); break;
 #endif
-	    case OPT_RESTART_DBOOTSTRAP: {  boxFinished(); exit (-1); }
+	    case OPT_RESTART_DBOOTSTRAP: 
+	      {  boxFinished();
+#if #cpu(s390)
+	      if(!NAME_ISDIR("/target/etc", &statbuf) && !fdisk_find_partition_by_mntpoint ("/target")) {
+              /* restore resolv.conf in case /target has been unmounted */
+	        execlog("rm -f /etc/resolv.conf", LOG_INFO);
+	        execlog("cp -a /tmp/resolv.conf /etc/resolv.conf", LOG_INFO);
+              }
+	      if(NAME_ISDIR("/lib/modules.old", &statbuf) && !fdisk_find_partition_by_mntpoint ("/target")) {
+	      /* restore old modules directory */   
+	        execlog("rm -rf /lib/modules", LOG_INFO);
+	        execlog("mv /lib/modules.old /lib/modules", LOG_INFO);
+	      }
+#endif
+	      exit (-1); }
+	    case OPT_PROBLEM_REPORT:   generate_problem_report(); break;
+	    case OPT_FOREIGN_MODULES:  add_modules_from_floppy(); break;
+#if (#cpu(s390))
+	    case OPT_ATTACH_DISK:
+		attach_disk(); break;
+#endif
 	    default: assert(0);
 	}
 #else  // _TESTING_
