@@ -16,7 +16,9 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <ctype.h> 
+#include <ctype.h>
+#include <resolv.h>
+#include <stdlib.h>
 #include "dbootstrap.h"
 #include "lang.h"
 #include "util.h"
@@ -30,8 +32,15 @@
 #define NC_ETC_NETWORK_DIR "/etc/network"
 #define NC_INTERFACES_FILE "/etc/network/interfaces"
 #define NC_HOSTS_FILE      "/etc/hosts"
-#define NC_PCMCIA_DIR      "/etc/pcmcia"
-#define NC_PCMCIA_FILE     "/etc/pcmcia/network.opts"
+/* 
+  Use 2.2.x because dhclient forces -q (quiet) and I like logging.
+  Using dhclient would make adding 2.0.x support just a matter of 
+  including the file in scripts/rootdisk/SMALL_BASE_LIST_all.
+  Also, right now dhclient is not in SMALL_BASE_LIST_all either 
+  since it doesn't do anything and it takes (a little) space.
+*/
+/* #define NC_DHCLIENT_FILE   "/sbin/dhclient" */
+#define NC_DHCLIENT_FILE   "/sbin/dhclient-2.2.x"
 
 #define MAX_INTERFACES (16)
 #define IP4_ADDR_SZ (4)
@@ -55,6 +64,8 @@ ip4_addr_t netmask = {{255, 255, 255, 0}};
 ip4_addr_t network = {{192, 168, 1, 0}};
 ip4_addr_t broadcast = {{192, 168, 1, 255}};
 ip4_addr_t gateway = {{0, 0, 0, 0}};
+ip4_addr_t peer = {{0, 0, 0, 0}};
+int has_pointopoint = 0;
 int has_gw = 0;
 int has_pcmcia = 0;
 int use_dhcp = 0;
@@ -251,6 +262,9 @@ int check_ip4_nameserver(char *ns) {
 int get_host()
 {
   char *def = NULL;
+  static const char *valid_hostname_chars =
+	  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-";
+  int len;
 
   for (;;) {
     if(host)
@@ -259,34 +273,36 @@ int get_host()
       def = strdup("debian");
 
     snprintf(prtbuf, PRTBUFSIZE,
-             _("Every Debian system has a name - even if it is not on a network. This name is called its \"hostname\". You should now specify a hostname for your new system.\n\nIf your system is going to be on a network, you should talk to the administrator of the network before you choose a hostname. If not, you may choose any name you like.\n\nThe hostname must not contain dots or underscores, and you must not append the domain name here.\n\nIf you can't think of a hostname for your new system, you may press <ENTER> to use the default hostname of \"%s\".\nPlease enter your Host name"), def);
+             _("Every Debian system has a name - even if it is not on a network. This name is called its \"hostname\". You should now specify a hostname for your new system.\n\nIf your system is going to be on a network, you should talk to the administrator of the network before you choose a hostname. If not, you may choose any name you like.\n\nThe hostname must not contain dots or underscores, and you must not append the domain name here.\n\nIf you can't think of a hostname for your new system, you may press <ENTER> to use the default hostname of \"%s\".\nPlease enter your Hostname"), def);
 
-    host = inputBox(prtbuf, _("Choose the Host name"), def);
+    host = inputBox(prtbuf, _("Choose the Hostname"), def);
     free(def);
     def = NULL;
 
     if (host == NULL)
       return 255;
 
-    if (strlen(host) == 0) {
+    if ((len = strlen(host)) == 0) {
       free(host);
       host = NULL;
       return 255;
     }
 
-    if(strchr(host, ' ')) {
-      problemBox(_("The host name must be one word."), _("Problem"));
+    /* Check the hostname for RFC 1123 compliance.  */
+    if (!isalnum(host[0])) {
+      problemBox(_("The hostname must begin with an alphanumeric character."),
+		 _("Problem"));
+    } else if (host[strlen(host) - 1] == '-') {
+      problemBox(_("The hostname must not end in a minus sign."),
+		 _("Problem"));
+    } else if (host[strspn(host, valid_hostname_chars)] != '\0') {
+      problemBox(_("The hostname may only contain alphanumeric characters and the minus sign."),
+		 _("Problem"));
+    } else if (!(2 <= len && len <= 63)) {
+      problemBox(_("The hostname must contain between 2 and 63 characters."),
+		 _("Problem"));
     } else {
-      if (strchr(host, '.')) {
-        problemBox(_("The host name must not contain dots."), _("Problem"));
-      } else {
-        if (isdigit(*host)) {
-          problemBox(_("The host name contains leading numerical character(s).\nNote that the RFC does not allow this."),
-                     _("Problem"));
-        } else {
-          break;
-        }
-      }
+      break;
     }
   }
   return 0;
@@ -337,6 +353,11 @@ int get_netinterface()
     }
   }
 
+  /* check if this is a Point-To-Point interface */
+  has_pointopoint = 0;
+  snprintf(prtbuf, PRTBUFSIZE, "ifconfig %s | grep -q POINTOPOINT", netinterface);
+  if (execlog(prtbuf, LOG_INFO) == 0) has_pointopoint = 1;
+
   has_pcmcia = 0;
   if (netinterface && NAME_ISDIR("/proc/bus/pccard", &nc_statbuf)) {
     snprintf(prtbuf, PRTBUFSIZE, _("The interface %s (%s) seems to be a PCMCIA card. Is this correct?"), 
@@ -361,6 +382,10 @@ int get_netinterface()
  */
 int get_ifconfig() {
   int ret, ix;
+#if #cpu (s390)
+  /* consider netsetup information gathered during first installation step*/
+  char * env_gateway = getenv("ip_gateway");
+#endif
 
   /* Get the IP address */
   snprintf(prtbuf, PRTBUFSIZE,
@@ -370,30 +395,64 @@ int get_ifconfig() {
                     _("You entered an invalid IP address.  Ask your network administrator for the proper value."));
   if (ret == 255) return ret;
 
-  /* Get the netmask */
-  ret = ask_for_IP4(&netmask, _("What is your netmask?"),
+  if(!has_pointopoint) {
+    /* Get the netmask */
+    ret = ask_for_IP4(&netmask, _("What is your netmask?"),
                     _("Choose the Network Mask"),
 		    _("You entered an invalid netmask.  Ask your network administrator for the proper value."));
-  if (ret == 255) return ret;
+    if (ret == 255) return ret;
+  }
+  else {
+    /* netmask is 255.255.255.255 */
+    for (ix = 0; ix < IP4_ADDR_SZ; ix++) {
+      netmask.i[ix] = 255;
+    }
+  }
 
   /*
    * Generate the network, broadcast and default gateway address
    * Default gateway address is network with last number set to "1", 
    * or "2" if "1" is the local address.
    */
-  for (ix = 0; ix < IP4_ADDR_SZ; ix++) {
-    gateway.i[ix] = network.i[ix] = ipaddr.i[ix] & netmask.i[ix];
-    broadcast.i[ix] = (~netmask.i[ix] & 255) | ipaddr.i[ix];
+  if (!has_gw) {
+    for (ix = 0; ix < IP4_ADDR_SZ; ix++) {
+      gateway.i[ix] = network.i[ix] = ipaddr.i[ix] & netmask.i[ix];
+      broadcast.i[ix] = (~netmask.i[ix] & 255) | ipaddr.i[ix];
+    }
+    gateway.i[IP4_ADDR_SZ - 1] |=
+      (ipaddr.i[IP4_ADDR_SZ - 1] == (network.i[IP4_ADDR_SZ - 1] | 1)) ? 2 : 1;
   }
-  gateway.i[IP4_ADDR_SZ - 1] |=
-    (ipaddr.i[IP4_ADDR_SZ - 1] == (network.i[IP4_ADDR_SZ - 1] | 1)) ? 2 : 1;
+
+#if #cpu (s390)
+  /* use gateway from initial netsetup */
+  if(env_gateway)
+    sscanf(env_gateway, "%d.%d.%d.%d", &gateway.i[0], &gateway.i[1], &gateway.i[2], &gateway.i[3]);
+#endif
 
   /* Get the gateway */
   has_gw = 1;
-  ret = ask_for_IP4(&gateway, _("A gateway system is one that connects your network to other networks such as your company wide-area net or the Internet.\n\nIf there is no gateway in your subnet just leave this field blank."),
+  if(!has_pointopoint) {
+    ret = ask_for_IP4(&gateway, _("A gateway system is one that connects your network to other networks such as your company wide-area net or the Internet.\n\nIf there is no gateway in your subnet just leave this field blank."),
                     _("What is your IP gateway address?"),
                     _("You entered an invalid gateway address.  Ask your network administrator for the proper value."));
-  if (ret == 255) has_gw = 0;
+    if (ret == 255) has_gw = 0;
+  }
+  else {
+    /* don't overwrite peer; important at least for s390 */
+    if(peer.i[0] == 0) {
+      for (ix = 0; ix < IP4_ADDR_SZ; ix++) {
+        peer.i[ix] = gateway.i[ix];
+      }
+    } 
+    /* Get the peer */
+    ret = ask_for_IP4(&peer, _("You are configuring a point-to-point network. The peer address is also used as gateway address."),
+                    _("What is your IP peer address?"),
+                    _("You entered an invalid peer address.  Ask your network administrator for the proper value."));
+    if (ret == 255) return ret;
+    for (ix = 0; ix < IP4_ADDR_SZ; ix++) {
+      gateway.i[ix] = peer.i[ix];
+    }
+  }
 
   return 0;
 }
@@ -454,7 +513,7 @@ int get_dnsconfig() {
       }
     }
 
-    nameserver = inputBox(_("Please enter the IP addresses (not host names) of up to 3 name servers, separated by spaces. Do not use commas. The servers will be queried in the order in which you enter them.\n\nIf you don't want to use any name servers just leave this field blank."),
+    nameserver = inputBox(_("Please enter the IP addresses (not hostnames) of up to 3 name servers, separated by spaces. Do not use commas. The servers will be queried in the order in which you enter them.\n\nIf you don't want to use any name servers just leave this field blank."),
                           _("Choose the DNS Server Addresses"), def);
     free(def);
 
@@ -474,6 +533,7 @@ int get_dnsconfig() {
 
 
 
+#ifdef PCMCIA
 /*
  * Get the PCMCIA transceiver type
  */
@@ -496,6 +556,7 @@ int get_pcmcia_transceiver() {
   }
   return 0;
 }
+#endif /* PCMCIA */
 
 static int write_hostname() {
     FILE *p_file;
@@ -560,6 +621,7 @@ int write_common_network() {
   if ((p_file = fopen(target_path(NC_INTERFACES_FILE), "w"))) {
     fprintf(p_file, "# /etc/network/interfaces -- configuration file for ifup(8), ifdown(8)\n\n");
     fprintf(p_file, "# The loopback interface\n");
+    fprintf(p_file, "auto lo\n");
     fprintf(p_file, "iface lo inet loopback\n");
     fclose(p_file);
   } else {
@@ -571,70 +633,6 @@ int write_common_network() {
 
 
 
-/*
- * Write the PCMCIA network configuration (static and DHCP)
- */
-int write_pcmcia_network() {
-  FILE *p_file;
-  char *tmp, *ns, *aux;
-  const char* path;
-  int ix;
-
-  path = target_path(NC_PCMCIA_DIR);
-  if (check_dir(path) == -1) mkdir(path, 0755);
-      
-  if ((p_file = fopen(target_path(NC_PCMCIA_FILE), "w"))) {
-    fprintf(p_file, "# Network adapter configuration\n#\n# Automatically generated during the Debian installation\n#\n");
-    fprintf(p_file, "# The address format is \"scheme,socket,instance,hwaddr\".\n#\ncase \"$ADDRESS\" in\n*,*,*,*)\n");
-    fprintf(p_file, "\t# Transceiver selection, for some cards -- see 'man ifport'\n");
-    if (transceiver) {
-      fprintf(p_file, "\tIF_PORT=\"%s\"\n", transceiver);
-    } else {
-      fprintf(p_file, "\tIF_PORT=\"\"\n");
-    }
-    fprintf(p_file, "\t# Use /sbin/pump for BOOTP/DHCP? [y/n]\n");
-    if(use_dhcp) {
-      fprintf(p_file, "\tPUMP=\"y\"\n");
-    } else {
-      fprintf(p_file, "\tPUMP=\"n\"\n");
-      fprintf(p_file, "\t# Host's IP address, netmask, network address, broadcast address\n");
-      fprintf(p_file, "\tIPADDR=\"%s\"\n", IP4tostr(prtbuf, ipaddr));
-      fprintf(p_file, "\tNETMASK=\"%s\"\n", IP4tostr(prtbuf, netmask));
-      fprintf(p_file, "\tNETWORK=\"%s\"\n", IP4tostr(prtbuf, network));
-      fprintf(p_file, "\tBROADCAST=\"%s\"\n", IP4tostr(prtbuf, broadcast));
-      if (has_gw) {
-        fprintf(p_file, "\tGATEWAY=\"%s\"\n", IP4tostr(prtbuf, gateway));
-      } else {
-        fprintf(p_file, "\tGATEWAY=\"\"\n");
-      }
-      if (domain) {
-        fprintf(p_file, "\tDOMAIN=\"%s\"\n", domain);
-      }
-      if (nameserver) {
-        aux = ns = strdup(nameserver);
-        ix = 1;
-        while ((tmp = strchr(ns, ' ')) != NULL) {
-          *tmp = '\0';
-          tmp++;
-          fprintf(p_file, "\tDNS_%d=\"%s\"\n", ix, ns);
-          ns = tmp;
-          ix++;
-        }
-        fprintf(p_file, "\tDNS_%d=\"%s\"\n", ix, ns);
-        free(aux);
-      } else {
-        fprintf(p_file, "\tDNS_1=\"\"\n");
-      }
-    }
-    fprintf(p_file, "\t;;\nesac\n");
-    fclose(p_file);
-    return 0;
-  } else {
-    return 255;
-  }
-}
-
-
 /*
  * Write the static network configuration
  * NOTE: This function must not be called with netinterface == NULL
@@ -682,7 +680,7 @@ int write_static_network() {
     }
     fclose(p_file);
     /* if this is a live CD filesystem, symlink to /tmp/resolv.conf too */
-    if (is_cdrom_image() == 2) {
+    if (is_live_cd() == 2) {
 	unlink("/tmp/resolv.conf");
 	symlink(target_path(NC_RESOLV_FILE), "/tmp/resolv.conf");
     }
@@ -690,28 +688,28 @@ int write_static_network() {
     ret = 255;
   }
 
-  /*
-   * Only write /etc/network/interfaces if we have a non-PCMCIA interface.
-   */
-  if (has_pcmcia) {
-    ret = write_pcmcia_network();
-  } else {
-    if ((p_file = fopen(target_path(NC_INTERFACES_FILE), "a"))) {
-      fprintf(p_file, "\n# The first network card - this entry was created during the Debian installation\n");
-      fprintf(p_file, "# (network, broadcast and gateway are optional)\n");
-      fprintf(p_file, "iface %s inet static\n", netinterface);
-      fprintf(p_file, "\taddress %s\n", IP4tostr(prtbuf, ipaddr));
-      fprintf(p_file, "\tnetmask %s\n", IP4tostr(prtbuf, netmask));
+  if ((p_file = fopen(target_path(NC_INTERFACES_FILE), "a"))) {
+    fprintf(p_file, "\n# The first network card - this entry was created during the Debian installation\n");
+    fprintf(p_file, "# (network, broadcast and gateway are optional)\n");
+    if (!has_pcmcia)
+      fprintf(p_file, "auto %s\n", netinterface);
+    fprintf(p_file, "iface %s inet static\n", netinterface);
+    fprintf(p_file, "\taddress %s\n", IP4tostr(prtbuf, ipaddr));
+    fprintf(p_file, "\tnetmask %s\n", IP4tostr(prtbuf, netmask));
+    if (!has_pointopoint) {
       fprintf(p_file, "\tnetwork %s\n", IP4tostr(prtbuf, network));
       fprintf(p_file, "\tbroadcast %s\n", IP4tostr(prtbuf, broadcast));
-      if (has_gw) {
-        fprintf(p_file, "\tgateway %s\n", IP4tostr(prtbuf, gateway));
-      }
-      fclose(p_file);
     } else {
-      ret = 255;
+      fprintf(p_file, "\tpointopoint %s\n", IP4tostr(prtbuf, peer));
     }
+    if (has_gw) {
+      fprintf(p_file, "\tgateway %s\n", IP4tostr(prtbuf, gateway));
+    }
+    fclose(p_file);
+  } else {
+    ret = 255;
   }
+
   return ret;
 }
 
@@ -727,17 +725,14 @@ int write_dhcp_network() {
   ret = write_common_network();
   if (ret == 255) return ret;
 
-  /* Copy /etc/resolv.conf (created by pump) to the target system */
-  copyFile(NC_RESOLV_FILE, target_path(NC_RESOLV_FILE), 0, 0);
-
-  if (has_pcmcia) {
-    write_pcmcia_network();
-  } else {
-    p_file = fopen(target_path(NC_INTERFACES_FILE), "a");
-    fprintf(p_file, "\n# The first network card - this entry was created during the Debian installation\n");
-    fprintf(p_file, "iface %s inet dhcp\n", netinterface);
-    fclose(p_file);
-  }
+  /* Copy /etc/resolv.conf (created by dhcp) to the target system */
+  vexeclog (LOG_DEBUG, "cp %s %s", NC_RESOLV_FILE, target_path(NC_RESOLV_FILE));
+  p_file = fopen(target_path(NC_INTERFACES_FILE), "a");
+  fprintf(p_file, "\n# The first network card - this entry was created during the Debian installation\n");
+  if (!has_pcmcia)
+    fprintf(p_file, "auto %s\n", netinterface);
+  fprintf(p_file, "iface %s inet dhcp\n", netinterface);
+  fclose(p_file);
   return 0;
 }
 
@@ -764,11 +759,32 @@ int activate_static_network()
     ret = execlog("/sbin/ifconfig lo 127.0.0.1", LOG_INFO);
     if (!use_dhcp && !has_pcmcia && (ret==0) && netinterface) {
       char cmdline[MAXLINE];
+
+      /* Down the interface to clear old gateways.  */
+      snprintf(cmdline, MAXLINE, "/sbin/ifconfig %s down", netinterface);
+      execlog(cmdline, LOG_INFO);
+      
+      /* Kill any dhcp client daemon that may have been running if the
+	 user previously chose DHCP */
+      if (NAME_ISEXE("/sbin/pump", &nc_statbuf))      
+	execlog("/sbin/pump -k", LOG_DEBUG);      
+      else if (NAME_ISEXE(NC_DHCLIENT_FILE, &nc_statbuf)) {
+	char *name = strrchr(NC_DHCLIENT_FILE, '/');
+	if (name) {
+	  snprintf(cmdline, MAXLINE, "kill `pidof %s`", name+1);
+	  execlog(cmdline, LOG_DEBUG);
+	}
+      }
+
       snprintf(cmdline, MAXLINE-5, "/sbin/ifconfig %s %s netmask ",
                netinterface, IP4tostr(prtbuf, ipaddr));
       strncat(cmdline, IP4tostr(prtbuf, netmask), MAXLINE-strlen(cmdline)-5);
       strncat(cmdline, " broadcast ", MAXLINE-strlen(cmdline)-5);
       strncat(cmdline, IP4tostr(prtbuf, broadcast), MAXLINE-strlen(cmdline)-5);
+      if (has_pointopoint) {
+       strncat(cmdline, " pointopoint ", MAXLINE-strlen(cmdline)-5);
+       strncat(cmdline, IP4tostr(prtbuf, peer), MAXLINE-strlen(cmdline)-5);
+      }
       ret = execlog(cmdline, LOG_INFO);
       if (has_gw && (ret==0)) {
         snprintf(cmdline, MAXLINE-5, "/sbin/route add default gw %s metric 1",
@@ -789,7 +805,6 @@ int activate_static_network()
  */
 int configure_static_network() {
   int ret;
-  has_gw = 0;
   transceiver = strdup("auto");
 
   if (!netinterface) {
@@ -803,18 +818,20 @@ int configure_static_network() {
 
     ret = get_dnsconfig();
     if (ret == 255) return ret;
-
+#ifdef PCMCIA
     if (has_pcmcia) {
       ret = get_pcmcia_transceiver();
       if (ret == 255) return ret;
     }
+#endif
 
     write_static_network();
-
+#ifdef PCMCIA
     if (has_pcmcia && activate_pcmcia()) {
         problemBox(_("As soon as you configure the PCMCIA support, your network configuration will be automatically activated."),
                    _("PCMCIA Networking"));
     }
+#endif
   }
 
   ret = activate_static_network();
@@ -824,10 +841,19 @@ int configure_static_network() {
 
 int configure_hostname() {
     int ret;
+#if #cpu (s390)
+    /* consider netsetup information gathered during first installation step*/
+    char * env_host = getenv("ip_host");
+
+    if(env_host && !host) {
+      host = malloc(strlen(env_host)+1);
+      strcpy(host, env_host);
+      }
+#endif
 
     ret = get_host();
     if(ret == 255) return ret;
-    ret = write_hostname();
+    ret = write_common_network();
     return ret;
 }
 
@@ -838,6 +864,25 @@ int configure_hostname() {
 int configure_network() {
   int ret;
   struct stat hf;
+#if #cpu (s390)
+  /* consider netsetup information gathered during first installation step*/
+  char * env_netinterface = getenv("network_interface");
+  char * env_ipaddr = getenv("ip_addr");
+  char * env_netmask = getenv("ip_netmask");
+  char * env_network = getenv("ip_network");
+  char * env_broadcast = getenv("ip_broadcast");
+  char * env_peer = getenv("ip_peer");
+  char * env_gateway = getenv("ip_gateway");
+  char * env_nameserver = getenv("ip_dns");
+  char * env_host = getenv("ip_host");
+  char * env_domain = getenv("ip_search");
+
+  if(env_host && !host) {
+    host = malloc(strlen(env_host)+1);
+    strcpy(host, env_host);
+  }
+#endif
+
   use_dhcp = 0;
 
   /* TODO: we need to see if there is already a network configuration, and
@@ -853,10 +898,40 @@ int configure_network() {
   ret = get_netinterface();
   if (ret == 255) return ret;
 
+#if #cpu (s390)
+  if(env_netinterface && !strcmp(netinterface, env_netinterface)) {
+    if(env_domain && !domain) {
+      domain = malloc(strlen(env_domain)+1);
+      strcpy(domain, env_domain);
+    }
+    if(env_nameserver && !nameserver) {
+      nameserver = malloc(strlen(env_nameserver)+1);
+      strcpy(nameserver, env_nameserver);
+    }
+    if(env_ipaddr)
+      sscanf(env_ipaddr, "%d.%d.%d.%d", &ipaddr.i[0], &ipaddr.i[1], &ipaddr.i[2], &ipaddr.i[3]);
+    if(env_netmask)
+      sscanf(env_netmask, "%d.%d.%d.%d", &netmask.i[0], &netmask.i[1], &netmask.i[2], &netmask.i[3]);
+    if(env_network)
+      sscanf(env_network, "%d.%d.%d.%d", &network.i[0], &network.i[1], &network.i[2], &network.i[3]);
+    if(env_broadcast)
+      sscanf(env_broadcast, "%d.%d.%d.%d", &broadcast.i[0], &broadcast.i[1], &broadcast.i[2], &broadcast.i[3]);
+    if(env_gateway)
+      sscanf(env_gateway, "%d.%d.%d.%d", &gateway.i[0], &gateway.i[1], &gateway.i[2], &gateway.i[3]);
+    if(env_peer)
+      sscanf(env_peer, "%d.%d.%d.%d", &peer.i[0], &peer.i[1], &peer.i[2], &peer.i[3]);
+  }
+#endif
+
   /* DHCP is only supported on Ethernet */
   if (netinterface &&
       (strncmp(netinterface, "eth", 3) == 0) &&
-      (NAME_ISEXE("/sbin/pump", &nc_statbuf))) {
+#if #cpu (s390)
+      /* if !static during initial setup for s390 */ 
+      (!env_netinterface || strcmp(netinterface, env_netinterface) || !env_ipaddr) &&
+#endif
+      (NAME_ISEXE("/sbin/pump", &nc_statbuf) || 
+       NAME_ISEXE( NC_DHCLIENT_FILE, &nc_statbuf ))) {
     snprintf(prtbuf, PRTBUFSIZE, 
 	     _("Configuration for %s (%s)\n\nDo you want to use DHCP or BOOTP to automatically configure this interface? You'll need a DHCP or BOOTP server in the local network for this to work."), 
 	     netinterface, get_ifdsc(netinterface));
@@ -870,8 +945,27 @@ int configure_network() {
 #if 0
       execlog("/sbin/pump -k", LOG_DEBUG);
 #endif
-      snprintf(prtbuf, MAXLINE-5, "/sbin/pump -i %s -h %s", netinterface, host);
-      if (execlog(prtbuf, LOG_INFO)) {
+    /* Look for dhclient first, if we find it we'll use it. */
+      if (NAME_ISEXE( NC_DHCLIENT_FILE, &nc_statbuf ))  
+      {
+      /* Setup host info for dhclient (I suppose we need to do this?) */
+	FILE * fp = fopen( "/etc/dhclient.conf", "w" );
+	if ( NULL != fp )
+	{
+	    fprintf( fp, "interface \"%s\" {\n send host-name \"%s\";\n }\n", 
+		netinterface, host );
+	    fclose( fp );
+	}
+
+      	snprintf( prtbuf, MAXLINE-5, "%s -e -lf /tmp/dhcp.leases %s", 
+		NC_DHCLIENT_FILE, netinterface );
+      }
+
+    /* No dhclient, so default to pump. */
+      else 
+      	snprintf(prtbuf, MAXLINE-5, "/sbin/pump -i %s -h %s", 
+		netinterface, host);
+      if (execlog(prtbuf, LOG_INFO) || !is_interface_up(netinterface)) {
 	boxPopWindow();
         snprintf(prtbuf, PRTBUFSIZE, _("Automatic configuration using DHCP/BOOTP failed for %s (%s).\n\nDo you want to manually configure this interface?"), netinterface, get_ifdsc(netinterface));
         if (yesNoBox(prtbuf, _("Automatic Network Configuration")) == DLG_YES) {
@@ -893,6 +987,7 @@ int configure_network() {
   } else {
     ret = configure_static_network();
   }
+  res_init();
   return ret;
 }
 
