@@ -26,11 +26,13 @@
 #include <syslog.h> 
 #include "util.h"
 
+#ifdef PCMCIA
+
 /*
  * Module variables
  */
 #ifdef _TESTING_
-static char pcmcia_id[]="$Id: pcmcia.c,v 1.22 2000/09/30 06:56:31 aph Exp $";
+static char pcmcia_id[]="$Id: pcmcia.c,v 1.29 2001/12/27 15:14:51 ender Exp $";
 #endif
 static char*  controller        = NULL;
 static char*  pcic_opts         = NULL;
@@ -62,12 +64,12 @@ static char   pcmcia_configured = 0;
 int         configure_pcmcia (void);
 int         activate_pcmcia (void);
 
-static void input_parameter (char*, char*, char**);
+static int  input_parameter (char*, char*, char**);
 static int  _activate_pcmcia (void);
 static int  _cardmgr (char);
-static void pcmcia_serial_file (void);
-static void pcmcia_cdrom_file (void);
-static void pcmcia_conf_file (void);
+static int  pcmcia_serial_file (void);
+static int  pcmcia_cdrom_file (void);
+static int  pcmcia_conf_file (void);
 static int  write_files (void);
 static int  write_lowlevel_files (void);
 static int  write_serial_opts (void);
@@ -167,9 +169,36 @@ _modules (char action)
 		break;
 
 	case MODULES_RMMOD:
-                                /* FIXME: must remove serial_cs and
-                                   other modules which ds uses,
-                                   otherwise this will fail */
+		{
+			/* Get the list of modules that are using ds,
+			   and unload them all */
+			FILE *fp = fopen("/proc/modules", "r");
+			if (fp) {
+				char buf[256];
+				while (fgets(buf, sizeof(buf), fp)) {
+					if (!strncmp(buf, "ds ", 3)) {
+						char *p = strchr(buf, '[');
+						char *q = strchr(buf, ']');
+						if (p && q) {
+							p++;
+							*q = 0;
+							while (p) {
+								q = strchr(p, ' ');
+								if (q) {
+									*q = 0;
+									q++;
+								}
+								snprintf (prtbuf, PRTBUFSIZE, "/sbin/rmmod %s", p);
+								execlog (prtbuf, LOG_INFO);
+								p = q;
+							}
+						}
+						break;
+					}
+				}
+				fclose(fp);
+			}
+		}
 		execlog ("/sbin/rmmod ds", LOG_INFO);
 		snprintf (prtbuf, PRTBUFSIZE, "/sbin/rmmod %s", controller);
 		execlog (prtbuf, LOG_INFO);
@@ -198,7 +227,7 @@ _cardmgr (char action)
 		if ( NAME_ISREG ("/var/run/cardmgr.pid", &statbuf) ) {
 			INFOMSG ("Stopping cardmgr");
 			rtn = execlog ("kill -TERM $(cat /var/run/cardmgr.pid) 2>/dev/null; "
-				       "sleep 3", LOG_INFO);
+				       "sleep 6", LOG_INFO);
 		} else {
 			INFOMSG ("No /var/run/cardmgr.pid, cardmgr not needing to be stopped");
 		}
@@ -303,7 +332,7 @@ activate_pcmcia (void)
 	return rtn;
 }
 
-static void
+static int
 input_parameter (char *text, char *title, char **variable)
 {
 	char *def = NULL;
@@ -319,28 +348,31 @@ input_parameter (char *text, char *title, char **variable)
 	free (def);
 
 	if ((*variable) == NULL)
-		(*variable) = strdup("");
+		return -1;
+
+	return 0;
 }
 
-static void
+static int
 pcmcia_serial_file (void)
 {
 	for (;;) {
-		input_parameter(_("Link used to configure dialout devices"),
-				_("Serial Device Link"),
-				&serial_link);
+		if (input_parameter(_("Link used to configure dialout devices"),
+				    _("Serial Device Link"),
+				    &serial_link) < 0)
+			return -1;
 
-		input_parameter(_("Options passed onto setserial"),
-				_("Setserial Options"),
-				&serial_opts);
+		if (input_parameter(_("Options passed onto setserial"),
+				    _("Setserial Options"),
+				    &serial_opts) < 0)
+			return -1;
 
 		if (yesNoBox (_("Should we create an inittab entry for this port?"),
-			      _("Inittab"))
-		    != 0) {
+			      _("Inittab")) != 0 &&
+		    input_parameter (_("Which command should be used for inittab?"),
+				     _("Inittab"),
+				     &inittab) == 0) {
 			use_inittab = 1;
-			input_parameter (_("Which command should be used for inittab?"),
-					 _("Inittab"),
-					 &inittab);
 		}
 		else {
 			use_inittab = 0;
@@ -363,9 +395,11 @@ pcmcia_serial_file (void)
 		if (yesNoBox (prtbuf, _("Confirmation")) == DLG_YES)
 			break;
 	}
+
+	return 0;
 }
 
-static void
+static int
 pcmcia_cdrom_file (void)
 {
 	for (;;) {
@@ -391,16 +425,19 @@ pcmcia_cdrom_file (void)
 			cdrom_do_mount = strdup ("n");
 		}
 
-		input_parameter (_("CD-ROMs normally have an iso9660 filesystem.\nWhich filesystem type should I use to mount the CD-ROM?"),
-				 _("Filesystem Type"),
-				 &cdrom_fstype);
-		input_parameter (_("The CD-ROM should be mounted using which options?"),
-				 _("Mount options"),
-				 &cdrom_opts);
-		input_parameter (_("The Debian filesystem structure uses /cdrom as the directory to mount any CD-ROMs. "
-				   "Where would you like to have your CD-ROM mounted?"),
-				 _("Mount point"),
-				 &cdrom_mountpt);
+		if (input_parameter (_("CD-ROMs normally have an iso9660 filesystem.\nWhich filesystem type should I use to mount the CD-ROM?"),
+				     _("Filesystem Type"),
+				     &cdrom_fstype) < 0)
+			return -1;
+		if (input_parameter (_("The CD-ROM should be mounted using which options?"),
+				     _("Mount options"),
+				     &cdrom_opts) < 0)
+			return -1;
+		if (input_parameter (_("The Debian filesystem structure uses /cdrom as the directory to mount any CD-ROMs. "
+				       "Where would you like to have your CD-ROM mounted?"),
+				     _("Mount point"),
+				     &cdrom_mountpt) < 0)
+			return -1;
 
 		/*
 		 * Confirm values
@@ -415,9 +452,12 @@ pcmcia_cdrom_file (void)
 		if (yesNoBox (prtbuf, _("Confirmation")) == DLG_YES)
 			break;
 	}
+
+	return 0;
 }
 
-void pcmcia_conf_file (void)
+static int
+pcmcia_conf_file (void)
 {
 	for (;;) {
 		/*
@@ -443,18 +483,20 @@ void pcmcia_conf_file (void)
 		/*
 		 * PCIC_OPTS, CORE_OPTS, CARDMGR_OPTS
 		 */
-		input_parameter (_("In systems with particularly fast processors, timing parameters need to be adjusted using \"cmd_time=<time>\" to determine the length of the PCMCIA bus cycles. The Cirrus PD6729 PCI controller needs \"fast_pci\" if the PCI speed is greater than 25 MHz. Also, unusual settings from manufacturers require that you provide special options in some cases. If you have a Halikan NBD 486 system, it has a TCIC-2 controller at an unusual location: you'll need add \"tcic_base=0x02c0\". Some systems using Cirrus controllers, including the NEC Versa M, the BIOS put the i82365 controller in a special suspended state at system startup time. On these systems, you will need to add the option \"wakeup=1\".\n\nYou probably don't need to add any option at all.\nWould you like to specify any controller setting?"),
-				 _("Controller Options"),
-				 &pcic_opts);
+		if (input_parameter (_("In systems with particularly fast processors, timing parameters need to be adjusted using \"cmd_time=<time>\" to determine the length of the PCMCIA bus cycles. The Cirrus PD6729 PCI controller needs \"fast_pci\" if the PCI speed is greater than 25 MHz. Also, unusual settings from manufacturers require that you provide special options in some cases. If you have a Halikan NBD 486 system, it has a TCIC-2 controller at an unusual location: you'll need add \"tcic_base=0x02c0\". In some systems using Cirrus controllers, including the NEC Versa M, the BIOS put the i82365 controller in a special suspended state at system startup time. On these systems, you will need to add the option \"wakeup=1\".\n\nYou probably don't need to add any option at all.\nWould you like to specify any controller setting?"),
+				     _("Controller Options"),
+				     &pcic_opts) < 0)
+			return -1;
 
 		input_parameter (_("To change the memory speed used for accessing a card's Card Information Structure (CIS), some systems with fast bus clocks will need to increase \"cis_speed=<speed>\".\n\nIt is highly unlikely that you should need to use this.\nWould you like to specify any core options?"),
 				 _("Core Options"),
 				 &core_opts);
 
                                 /*  FIXME: why not skip this unless verbose ? */
-		input_parameter (_("\"cardmgr\" is the daemon in charge of managing PCMCIA devices. Sometimes you want it to run with special settings, for example to enable debugging functionality. You can specify the options here.\n\nAs before you shouldn't worry about this at all.\nWould you like to specify any cardmgr options?"),
-				 _("Card Manager Options"),
-				 &cardmgr_opts);
+		if (input_parameter (_("\"cardmgr\" is the daemon in charge of managing PCMCIA devices. Sometimes you want it to run with special settings, for example to enable debugging functionality. You can specify the options here.\n\nAs before you shouldn't worry about this at all.\nWould you like to specify any cardmgr options?"),
+				     _("Card Manager Options"),
+				     &cardmgr_opts) < 0)
+			return -1;
 
 		/*
 		 * Confirm values
@@ -469,6 +511,8 @@ void pcmcia_conf_file (void)
 		if (yesNoBox (prtbuf, _("Confirmation")) == DLG_YES)
 			break;
 	}
+
+	return 0;
 }
 
 static int
@@ -524,25 +568,26 @@ write_lowlevel_files (void)
 	}
 	WRT_HDR (p_file);
 	fprintf (p_file, "PCMCIA='yes'\n");
-	fprintf (p_file, _("#\n"
-			   "# Should be either i82365 or tcic\n"
-			   "#\n"
-			   "PCIC=\"%s\"\n"
-			   "\n"
-			   "#\n"
-			   "# Put socket driver timing parameters here.\n"
-			   "#\n"
-			   "PCIC_OPTS=\"%s\"\n"
-			   "\n"
-			   "#\n"
-			   "# Put pcmcia_core options here.\n"
-			   "#\n"
-			   "CORE_OPTS=\"%s\"\n"
-			   "\n"
-			   "#\n"
-			   "# Put cardmgr options here.\n"
-			   "#\n"
-			   "CARDMGR_OPTS=\"%s\"\n"),
+	fprintf (p_file, 
+                 "#\n"
+                 "# Should be either i82365 or tcic\n"
+                 "#\n"
+                 "PCIC=\"%s\"\n"
+                 "\n"
+                 "#\n"
+                 "# Put socket driver timing parameters here.\n"
+                 "#\n"
+                 "PCIC_OPTS=\"%s\"\n"
+                 "\n"
+                 "#\n"
+                 "# Put pcmcia_core options here.\n"
+                 "#\n"
+                 "CORE_OPTS=\"%s\"\n"
+                 "\n"
+                 "#\n"
+                 "# Put cardmgr options here.\n"
+                 "#\n"
+                 "CARDMGR_OPTS=\"%s\"\n",
 		 controller,
 		 pcic_opts,
 		 core_opts,
@@ -580,28 +625,24 @@ write_serial_opts (void)
 	fprintf (p_file, SERIAL_TXT);
 
 	fprintf (p_file,
-		 _("#\n"
-		   "# The address format is \"scheme,socket,instance\".\n"
-		   "#\n"));
-
-	fprintf (p_file,
+		 "#\n"
+                 "# The address format is \"scheme,socket,instance\".\n"
+                 "#\n"
 		 "case \"$ADDRESS\" in\n"
-		 "*,*,*)\n");
-
-	fprintf (p_file,
-		 _("\t#\n"
-		   "\t#Symbolic link to dialout device\n"
-		   "\t#\n"
-		   "\tLINK=\"%s\"\n"
-		   "\n"
-		   "\t#\n"
-		   "\t# Options for 'setserial'\n"
-		   "\t#\n"
-		   "\tSERIAL_OPTS=\"%s\"\n"
-		   "\n"
-		   "\t#\n"
-		   "\t# Should we create an inittab entry for this port?\n"
-		   "\t#\n"),
+		 "*,*,*)\n"
+		 "\t#\n"
+                 "\t#Symbolic link to dialout device\n"
+                 "\t#\n"
+                 "\tLINK=\"%s\"\n"
+                 "\n"
+                 "\t#\n"
+                 "\t# Options for 'setserial'\n"
+                 "\t#\n"
+                 "\tSERIAL_OPTS=\"%s\"\n"
+                 "\n"
+                 "\t#\n"
+                 "\t# Should we create an inittab entry for this port?\n"
+                 "\t#\n",
 		 serial_link,
 		 serial_opts);
 
@@ -634,26 +675,22 @@ write_cdrom_opts (void)
 	fprintf (p_file, CDROM_TXT);
 
 	fprintf(p_file,
-	  _("#\n"
-	    "# The address format is \"scheme,socket\".\n"
-	    "#\n"));
-
-	fprintf (p_file,
-		 "case \"$ADDRESS\" in\n"
-		 "*,*)\n");
-
-	fprintf (p_file,
-		 "\tDO_FSTAB=\"%s\" ; DO_MOUNT=\"%s\"\n"
-		 "\tFSTYPE=\"%s\"\n"
-		 "\tOPTS=\"%s\"\n"
-		 "\tMOUNTPT=\"%s\"\n"
-		 "\t;;\n"
-		 "esac\n",
-		 cdrom_do_fstab,
-		 cdrom_do_mount,
-		 cdrom_fstype,
-		 cdrom_opts,
-		 cdrom_mountpt);
+                "#\n"
+                "# The address format is \"scheme,socket\".\n"
+                "#\n"
+                "case \"$ADDRESS\" in\n"
+                "*,*)\n"
+                "\tDO_FSTAB=\"%s\" ; DO_MOUNT=\"%s\"\n"
+                "\tFSTYPE=\"%s\"\n"
+                "\tOPTS=\"%s\"\n"
+                "\tMOUNTPT=\"%s\"\n"
+                "\t;;\n"
+                "esac\n",
+                cdrom_do_fstab,
+                cdrom_do_mount,
+                cdrom_fstype,
+                cdrom_opts,
+                cdrom_mountpt);
 
 	fclose(p_file);
 
@@ -716,11 +753,17 @@ configure_pcmcia (void)
 				switch (ix)
 				{
 				case 0:
-					pcmcia_conf_file(); break;
+					if (pcmcia_conf_file() < 0)
+						return 255;
+					break;
 				case 1:
-					pcmcia_serial_file(); break;
+					if (pcmcia_serial_file() < 0)
+						return 255;
+					break;
 				case 2:
-					pcmcia_cdrom_file(); break;
+					if (pcmcia_cdrom_file() < 0)
+						return 255;
+					break;
 #if 0
 				case 3:
 					pcmcia_network_file(); break;
@@ -775,3 +818,5 @@ main (void)
  * c-file-style: "linux"
  * End:
  */
+
+#endif /* PCMCIA */
