@@ -18,9 +18,44 @@
 #include <syslog.h>
 #include <termios.h>
 #include <unistd.h>
+#include <slang.h>
+#include <newt.h>
+
+/* for __NR_sysfs */
+#include <asm/unistd.h>
 
 #ifdef USE_LANGUAGE_CHOOSER
 #include <wchar.h>
+
+char *mirror_list[] = { 
+"at", "au", "bg", "br", "cz", "de", "dk", "ee", "es", "fi", "fr", "uk", "hk",
+"hr", "hu", "it", "jp", "nl", "no", "nz", "pl", "ru", "se", "si", "tr", "us",
+NULL };
+
+
+char *mirror_country_map[] = { 
+/*   "en", "us", */
+   "ca", "es", /*
+   "cs", "us",
+   "da", "us", */
+   "de", "de",
+/*   "eo", "us", */
+   "es", "es",
+   "fi", "fi",
+   "fr", "fr",
+   "gl", "es",
+   "hr", "hr",
+   "hu", "hu",
+   "it", "it",
+   "ja", "jp",
+   "pl", "pl",
+/*   "pt", "us", */
+   "ru", "ru",
+/*   "sk", "us",
+   "sv", "us", */
+   "tr", "tr",
+   NULL };
+
 #endif
 
 #include "dbootstrap.h"
@@ -55,6 +90,16 @@ int execlog (const char *incmd, int priority)
   return(pclose(output)); 
 }
 
+int
+vexeclog(int priority, const char *fmt, ...) {
+    va_list ap;
+
+    va_start (ap, fmt);
+    vsnprintf (prtbuf, sizeof (prtbuf) - 1, fmt, ap);
+    va_end (ap);
+
+    return execlog (prtbuf, priority);
+}
 
 int fullscreen_execlog  (const char *cmd)
 {
@@ -62,8 +107,9 @@ int fullscreen_execlog  (const char *cmd)
 
   DEBUGMSG("running cmd fullscreen '%s'", cmd);
 
+  newtCls();
+  SLsmg_gotorc(0, 0);
   boxSuspend();
-  printf(CLEAR);
   ret = system(cmd);
   boxResume();
 
@@ -74,6 +120,176 @@ int fullscreen_execlog  (const char *cmd)
   return(ret);
 }
 
+int execlog_to_string(const char *cmd, int *exitcode,
+		      char *result, int len, int flag)
+{
+  int retval = 0;
+  int stdoutpipe[2];
+  int stderrpipe[2];
+  int childpid;
+  int newlen = 0;
+  char buf[1024];
+  int stdout_eof = 0;
+  int stderr_eof = 0;
+  
+  fd_set readfdset;
+  
+  if (pipe(stdoutpipe))
+    {
+      ERRMSG(_("Couldn't create pipe: %s"), strerror(errno));
+      return -1;
+    }
+
+  if (flag != 0)
+    if (pipe(stderrpipe))
+      {
+	ERRMSG(_("Couldn't create pipe: %s"), strerror(errno));
+	return -1;
+      }
+  
+  if ((childpid = fork()) < 0)
+    {
+      ERRMSG(_("Couldn't fork: %s"), strerror(errno));
+      return -1;
+    }
+  else if (childpid == 0)
+    {
+      /* child */
+      close(stdoutpipe[0]);
+      if (flag != 0)
+	close(stderrpipe[0]);
+      dup2(stdoutpipe[1], 1);
+      close(stdoutpipe[1]);
+      if (flag != 0)
+	{
+	  dup2(stderrpipe[1], 2);
+	  close(stderrpipe[1]);
+	}
+      else
+	{
+	  int fd;
+	  if ((fd = open("/dev/null", O_WRONLY)) < 0)
+	    {
+	      ERRMSG(_("Couldn't open /dev/null: %s"), strerror(errno));
+	      exit(1);
+	    }
+	  dup2(fd, 2);
+	  close(fd);
+	}
+      if (execl("/bin/sh", "/bin/sh", "-c", cmd, NULL))
+	{
+	  ERRMSG(_("Couldn't run /bin/sh -c '%s': %s"), cmd, strerror(errno));
+	  exit(1);
+	}
+    }
+  /* parent */
+  close(stdoutpipe[1]);
+  if (flag != 0)
+    close(stderrpipe[1]);
+
+  if (bootargs.isdebug) {
+    openlog(SYSLOG_IDENT, LOG_PID, LOG_USER);
+    syslog(LOG_DEBUG, "running cmd '%s'", cmd);
+  }
+  else {
+    openlog(SYSLOG_IDENT, LOG_PID, LOG_USER);
+  }
+
+  /* save space for the null terminator */
+  if (--len <= 0)
+    {
+      retval = -1;
+      goto done;
+    }
+
+  while (1)
+    {
+      int status;
+      int cur;
+
+      FD_ZERO(&readfdset);
+      if (!stdout_eof)
+	FD_SET(stdoutpipe[0], &readfdset);
+      if (flag != 0 && !stderr_eof)
+	FD_SET(stderrpipe[0], &readfdset);
+
+      /* If there is nothing to read, and the process has exited, just
+	 return the exit status. */
+      if ((stdout_eof && (stderr_eof || flag == 0))
+	  || (select(FD_SETSIZE, &readfdset, NULL, NULL, NULL) == 0
+	      && errno != EINTR))
+	{
+	  waitpid(childpid, &status, 0);
+	  if (WIFEXITED(status))
+	    *exitcode = WEXITSTATUS(status);
+	  else if (WIFSIGNALED(status) || WCOREDUMP(status))
+	    *exitcode = 1;
+	  result[newlen] = '\0';
+	  retval = 0;
+	  goto done;
+	}
+
+      /* Check stdout */
+      if (FD_ISSET(stdoutpipe[0], &readfdset))
+	{
+	  if ((cur = read(stdoutpipe[0], result+newlen,
+			  len-newlen < 0 ? 0 : len-newlen)) < 0)
+	    {
+	      retval = -1;
+	      goto done;
+	    }
+	  else if (cur == 0)
+	    {
+	      stdout_eof = 1;
+	      close(stdoutpipe[0]);
+	    }
+	  else
+	    newlen += cur;
+	}
+      /* Check stderr */
+      if (flag != 0 && FD_ISSET(stderrpipe[0], &readfdset))
+	{
+	  if (flag == 1)
+	    {
+	      if ((cur = read(stderrpipe[0], buf, sizeof(buf)-1)) < 0)
+		{
+		  retval = -1;
+		  goto done;
+		}
+	      else if (cur == 0)
+		{
+		  stderr_eof = 1;
+		  close(stderrpipe[0]);
+		}
+	      else
+		{
+		  buf[cur] = '\0';
+		  syslog(LOG_ERR, buf[cur-1] == '\n' ? "%s" : "%s\n", buf);
+		}
+	    }
+	  else if (flag == 2)
+	    {
+	      if ((cur = read(stderrpipe[0], result+newlen,
+			      len-newlen < 0 ? 0 : len-newlen)) < 0)
+		{
+		  retval = -1;
+		  goto done;
+		}
+	      else if (cur == 0)
+		{
+		  stderr_eof = 1;
+		  close(stderrpipe[0]);
+		}
+	      else
+		newlen += cur;
+	    }
+	}
+    }
+ done:
+  closelog();
+  return retval;
+}
+
 
 int check_target_path(void)
 {
@@ -129,7 +345,11 @@ int write_userconfig (const char *key, const char *value)
   if ((conffile = fopen(target_path(USER_SETTINGS_FILE), "a")) == NULL)
       return 1;
 
-  fprintf(conffile, "%s='%s'\n", key, value);
+  if (value)
+      fprintf(conffile, "%s='%s'\n", key, value);
+  else
+      fprintf(conffile, "%s=''\n", key);
+
   fclose(conffile);
   
   return 0;
@@ -224,6 +444,84 @@ is_network_configured(void)
     return 0;
 }
 
+inline int
+is_filesystem_supported(const char *fstype)
+{
+  return (syscall(__NR_sysfs, 1, fstype) < 0);
+}
+/*
+char *
+fs_modules()
+{
+  FILE *proc;
+  int i=0;
+  char *space, *list, line[32];
+  
+  proc=fopen("/proc/modules","r");
+  if ( proc ==  NULL )
+    {
+      perrorBox(_("Can't read /proc/modules"));
+      return NULL;
+    }
+  while(!feof(proc)) {
+    fgets(line, 31, proc);
+    space = (strchr(line, ' '));
+    if(space != NULL) 
+    {
+      *space = '\0'; 
+      if(!strcmp(line, "reiserfs"))
+          addtolist(&list, line);
+      if(!strcmp(line, "ext3"))
+          addtolist(&list, line);
+      if(!strcmp(line, "xfs"))
+          addtolist(&list, line);
+      if(!strcmp(line, "umsdos"))
+          addtolist(&list, line);
+      if(!strcmp(line, "ext2"))
+          addtolist(&list, line);
+      if(!strcmp(line, "minix"))
+          addtolist(&list, line);
+    }
+  }
+  return list;
+}
+*/
+
+int
+is_fstype (const char *mount, const char *fs)
+{
+  FILE *proc;
+  int match=0;
+  char dev[80], dir[80], type[80];
+
+  proc = fopen ("/proc/mounts", "r");
+  if ( proc ==  NULL )
+    {
+      perrorBox(_("Can't read /proc/mounts"));
+      return -1;
+    }
+
+  while ( EOF != fscanf (proc, "%s %s %s %*s %*i %*i\n",
+			 dev, dir, type))
+    {
+      if (!strncmp (mount, dev, 79))
+	{
+	  match = strncmp (fs, type, 79) ? 0 : 1;
+	  break;
+	}
+
+      if (!strncmp (mount, dir, 79))
+	{
+	  match = strncmp (fs, type, 79) ? 0 : 1;
+	  break;
+	}
+
+    }
+  fclose (proc);
+
+  return match;
+}
+
 int
 is_interface_up(char *inter)
 {
@@ -343,18 +641,16 @@ initchroot_cap(void)
 
 /* This may get more complex...leave it simple for now */
 int
-is_cdrom_image(void)
+is_live_cd(void)
 {
-    struct stat statbuf;
-    if (NAME_ISREG("/cdrom_image",&statbuf) || bootargs.cdrom) {
-	if (NAME_ISDIR("/dists",&statbuf))
-	    return 2;
-	else
-	    return 1;
-    } else
-	return 0;
+   struct stat statbuf;
+   if (NAME_ISDIR("/dists",&statbuf))
+      return 2;
+   else
+      return 0;
 }
 
+
 int
 check_pending_config(void)
 {
@@ -369,7 +665,7 @@ check_pending_config(void)
      */
     execlog("cp -pR " TARGET_NOPREFIX "/. " TARGET_PREFIX, LOG_INFO);
     /* also, readjust the symlink if needed for cdrom filesystem */
-    if (is_cdrom_image() == 2) {
+    if (is_live_cd() == 2) {
 	unlink("/tmp/resolv.conf");
 	symlink(target_path("/etc/resolv.conf"), "/tmp/resolv.conf");
     }
@@ -402,12 +698,78 @@ check_pending_config(void)
     write_userconfig("VERBOSE", "quiet");
   if ( bootargs.isdebug )
     write_userconfig("DEBUG", "true");
-  if ( bootargs.cdrom )
+/*fixme later, was ist dieses write_userconfig  if ( is_live_cd() > 0 )
     write_userconfig("CDROM", "true");
+*/
 
 #ifdef USE_LANGUAGE_CHOOSER
-  if ( lang != NULL )
-    write_userconfig("LANGUAGE", lang);
+  if ( lang != NULL ) {
+    /* The locale should be generated by base-config if needed, and ignored
+       if it can't be used. */
+    write_userconfig("LANG_INST", lang->locale);
+
+    /* Specify priority list of translations to use when displaying messages */
+    write_userconfig("LANGUAGE_INST", lang->msgcat);
+  }
+#endif
+
+#if #cpu(powerpc)
+  if ( ( 0 == strcmp( Arch2, "PowerMac" ) ) && ( NULL != Arch3 ) && 
+	( 0 == strcmp( Arch3, "NewWorld" ) ) ) {
+    struct fdisk_partition *bootstrap;
+    char *message = NULL, *title = NULL;
+
+    bootstrap = fdisk_find_partition_by_type( PTYPE_MAC_BOOT );
+    if ( NULL == bootstrap ) {
+	message = _("No Apple_Bootstrap partition was detected.");
+	title = _("Missing Bootstrap Partition");
+    }
+    else if ( 800 > bootstrap->size ) {
+	message = _("The Apple_Bootstrap partition must be at least 800K.");
+	title = _("Bootstrap Partition Too Small");
+    }
+    else if ( 0 != strcmp( bootstrap->disk->name, Root->disk->name ) ) {
+	message = _("The Apple_Bootstrap partition must reside on the same disk "
+		      "as the root partition.");
+	title = _("No Bootstrap Partition On Root Disk");
+    }
+
+    if (message != NULL && title != NULL)
+    {
+        snprintf (prtbuf, sizeof (prtbuf), _("%s  You should repartition your disk.\n\n"
+		      "If you do not repartition and add an 800K Apple_Bootstrap "
+		      "partition you will not be able to make the disk bootable."), message);
+
+        problemBox (prtbuf, title);
+    }
+  } 
+  else if ( 0 == strcmp( Arch2, "chrp" ) ) {
+    struct fdisk_partition *bootstrap;
+    char *message = NULL, *title = NULL;
+
+    bootstrap = fdisk_find_partition_by_type( PTYPE_CHRP_BOOT );
+    if ( NULL == bootstrap ) {
+	message = _("No CHRP bootstrap partition was detected.");
+	title = _("Missing Bootstrap Partition");
+    }
+    else if ( 800 > bootstrap->size ) {
+	message = _("The CHRP bootstrap partition must be at least 800K.");
+	title =_("Bootstrap Partition Too Small");
+    }
+    else if ( 0 != strcmp( bootstrap->disk->name, Root->disk->name ) ) {
+        message = _("The CHRP bootstrap partition must reside on the same disk as the root partition.");
+        title = _("No Bootstrap Partition On Root Disk");
+    }
+
+    if (message != NULL && title != NULL)
+    {
+        snprintf (prtbuf, sizeof (prtbuf), _("%s  You should repartition your disk.\n\n"
+		      "If you do not repartition and add an 800K type 0x41 "
+		      "partition you will not be able to make the disk bootable."), message);
+
+        problemBox (prtbuf, title);
+    }
+  }
 #endif
 
   return 0;
@@ -447,7 +809,7 @@ do_interruptable (const char *message,
 		  void (*thunk) (void *result),
 		  void *result)
 {
-  int saved_errno;
+  int saved_errno = 0;
   const char *press_cc = _("  Press Ctrl-C to interrupt.");
   char msg_buf[ strlen (message) + strlen (press_cc) + 1 ];
   snprintf (msg_buf, sizeof (msg_buf), "%s%s", message, press_cc);
@@ -517,163 +879,6 @@ int main (int argc, char **argv) {
  */
 
 
-/*
- * Return TRUE if a fileName is a directory.
- * Nonexistant files return FALSE.
- */
-int isDirectory(const char *name)
-{
-    struct stat statBuf;
-
-    if (stat(name, &statBuf) < 0)
-	return FALSE;
-    if (S_ISDIR(statBuf.st_mode))
-	return TRUE;
-    return(FALSE);
-}
-
-
-/*
- * Copy one file to another, while possibly preserving its modes, times,
- * and modes.  Returns TRUE if successful, or FALSE on a failure with an
- * error message output.  (Failure is not indicted if the attributes cannot
- * be set.)
- *
- * Code written by Erik Andersen <andersee@debian.org> and
- * swiped from busybox.
- */
-int
-copyFile( const char *srcName, const char *destName, 
-	 int setModes, int followLinks)
-{
-    int rfd;
-    int wfd;
-    int rcc;
-    int result;
-    char buf[BUF_SIZE];
-    struct stat srcStatBuf;
-    struct stat dstStatBuf;
-    struct utimbuf times;
-
-    if (followLinks == FALSE)
-	result = stat(srcName, &srcStatBuf);
-    else 
-	result = lstat(srcName, &srcStatBuf);
-    if (result < 0) {
-	perror(srcName);
-	return FALSE;
-    }
-
-    if (followLinks == FALSE)
-	result = stat(destName, &dstStatBuf);
-    else 
-	result = lstat(destName, &dstStatBuf);
-    if (result < 0) {
-	dstStatBuf.st_ino = -1;
-	dstStatBuf.st_dev = -1;
-    }
-
-    if ((srcStatBuf.st_dev == dstStatBuf.st_dev) &&
-	(srcStatBuf.st_ino == dstStatBuf.st_ino)) {
-	fprintf(stderr, "Copying file \"%s\" to itself\n", srcName);
-	return FALSE;
-    }
-
-    if (S_ISDIR(srcStatBuf.st_mode)) {
-	//fprintf(stderr, "copying directory %s to %s\n", srcName, destName);
-	/* Make sure the directory is writable */
-	result = mkdir(destName, 0777777 ^ umask(0));
-	if (result < 0 && errno != EEXIST) {
-	    perror(destName);
-	    return (FALSE);
-	}
-    } else if (S_ISLNK(srcStatBuf.st_mode)) {
-	char *link_val;
-	int link_size;
-
-	//fprintf(stderr, "copying link %s to %s\n", srcName, destName);
-	link_val = (char *) alloca(PATH_MAX + 2);
-	link_size = readlink(srcName, link_val, PATH_MAX + 1);
-	if (link_size < 0) {
-	    perror(srcName);
-	    return (FALSE);
-	}
-	link_val[link_size] = '\0';
-	link_size = symlink(link_val, destName);
-	if (link_size != 0) {
-	    perror(destName);
-	    return (FALSE);
-	}
-    } else if (S_ISFIFO(srcStatBuf.st_mode)) {
-	//fprintf(stderr, "copying fifo %s to %s\n", srcName, destName);
-	if (mkfifo(destName, 0644)) {
-	    perror(destName);
-	    return (FALSE);
-	}
-    } else if (S_ISBLK(srcStatBuf.st_mode) || S_ISCHR(srcStatBuf.st_mode) 
-	    || S_ISSOCK (srcStatBuf.st_mode)) {
-	//fprintf(stderr, "copying soc, blk, or chr %s to %s\n", srcName, destName);
-	if (mknod(destName, srcStatBuf.st_mode, srcStatBuf.st_rdev)) {
-	    perror(destName);
-	    return (FALSE);
-	}
-    } else if (S_ISREG(srcStatBuf.st_mode)) {
-	//fprintf(stderr, "copying regular file %s to %s\n", srcName, destName);
-	rfd = open(srcName, O_RDONLY);
-	if (rfd < 0) {
-	    perror(srcName);
-	    return FALSE;
-	}
-
-	wfd = creat(destName, srcStatBuf.st_mode);
-	if (wfd < 0) {
-	    perror(destName);
-	    close(rfd);
-	    return FALSE;
-	}
-
-	while ((rcc = read(rfd, buf, sizeof(buf))) > 0) {
-	    if (fullWrite(wfd, buf, rcc) < 0)
-		goto error_exit;
-	}
-	if (rcc < 0) {
-	    goto error_exit;
-	}
-
-	close(rfd);
-	if (close(wfd) < 0) {
-	    return FALSE;
-	}
-    }
-
-    if (setModes == TRUE) {
-	//fprintf(stderr, "Setting permissions for %s\n", destName);
-	chmod(destName, srcStatBuf.st_mode);
-#if (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 1)
-	if (followLinks == FALSE)
-	    lchown(destName, srcStatBuf.st_uid, srcStatBuf.st_gid);
-	else
-#endif
-	    chown(destName, srcStatBuf.st_uid, srcStatBuf.st_gid);
-
-	times.actime = srcStatBuf.st_atime;
-	times.modtime = srcStatBuf.st_mtime;
-
-	utime(destName, &times);
-    }
-
-    return TRUE;
-
-
-  error_exit:
-    perror(destName);
-    close(rfd);
-    close(wfd);
-
-    return FALSE;
-}
-
-
 
 /*
  * Walk down all the directories under the specified 
@@ -703,7 +908,7 @@ recursiveAction(const char *fileName, int recurse, int followLinks, int depthFir
 	status = lstat(fileName, &statbuf);
 
     if (status < 0) {
-	perror(fileName);
+	ERRMSG("Cannot stat %s", fileName);
 	return (FALSE);
     }
 
@@ -725,7 +930,7 @@ recursiveAction(const char *fileName, int recurse, int followLinks, int depthFir
     
     status = lstat(fileName, &statbuf1);
     if (status < 0) {
-	perror(fileName);
+	ERRMSG("Cannot stat %s", fileName);
 	return (FALSE);
     }
 
@@ -733,20 +938,22 @@ recursiveAction(const char *fileName, int recurse, int followLinks, int depthFir
 	DIR *dir;
 	dir = opendir(fileName);
 	if (!dir) {
-	    perror(fileName);
+	    ERRMSG("Cannot open %s", fileName);
 	    return (FALSE);
 	}
 	if (dirAction != NULL && depthFirst == FALSE) {
 	    status = dirAction(fileName, &statbuf);
 	    if (status == FALSE) {
-		perror(fileName);
+		ERRMSG("DirAction not successful with %s", fileName);
 		return (FALSE);
 	    }
 	}
 	while ((next = readdir(dir)) != NULL) {
 	    char nextFile[NAME_MAX];
-	    if ((strcmp(next->d_name, "..") == 0)
-		|| (strcmp(next->d_name, ".") == 0)) {
+	    if ((!strcmp(next->d_name, ".."))
+		|| (!strcmp(next->d_name, "."))
+		|| (!strcmp(next->d_name, ".resource")) /* .resource and .finderinfo are HFS cruft */
+		|| (!strcmp(next->d_name, ".finderinfo"))) {
 		continue;
 	    }
 	    sprintf(nextFile, "%s/%s", fileName, next->d_name);
@@ -760,13 +967,13 @@ recursiveAction(const char *fileName, int recurse, int followLinks, int depthFir
 	}
 	status = closedir(dir);
 	if (status < 0) {
-	    perror(fileName);
+	    ERRMSG("Cannot close directory");
 	    return (FALSE);
 	}
 	if (dirAction != NULL && depthFirst == TRUE) {
 	    status = dirAction(fileName, &statbuf);
 	    if (status == FALSE) {
-		perror(fileName);
+		ERRMSG("DirAction failed for %s", fileName);
 		return (FALSE);
 	    }
 	}
@@ -780,40 +987,11 @@ recursiveAction(const char *fileName, int recurse, int followLinks, int depthFir
 }
 
 
-
-
-/*
- * Write all of the supplied buffer out to a file.
- * This does multiple writes as necessary.
- * Returns the amount written, or -1 on an error.
- */
-int fullWrite(int fd, const char *buf, int len)
-{
-    int cc;
-    int total;
-
-    total = 0;
-
-    while (len > 0) {
-	cc = write(fd, buf, len);
-
-	if (cc < 0)
-	    return -1;
-
-	buf += cc;
-	total += cc;
-	len -= cc;
-    }
-
-    return total;
-}
-
 /* Is the target an NFS mountpoint? */
 int
 is_nfs_partition( const char* target )
 {
     struct fdisk_partition* p;
-    mounted_reread();
     p = fdisk_find_partition_by_mntpoint(target);
     if ((p) && (p->type == PTYPE_NFS))
 	return 1;
@@ -833,26 +1011,6 @@ int is_root_a_floppy (void) {
     return 0;
 }
 
-int no_match(const char *path, const char *name, __mode_t mode){
-  int i,status;
-  glob_t globbuf;
-  struct stat statbuf;
-  snprintf(prtbuf,PRTBUFSIZE,"%s/*/%s",path,name);
-  glob( prtbuf, GLOB_NOSORT, NULL, &globbuf);
-  status=1;
-  if ( globbuf.gl_pathc > 0 ) {
-    for (i=0;i<globbuf.gl_pathc;i++) {
-      if ( (! stat(globbuf.gl_pathv[i],&statbuf) ) &&
-           ( (statbuf.st_mode & S_IFMT) == mode ) )  {
-        status=0;
-        break;
-      }
-    }
-  }
-  globfree(&globbuf); 
-  return status;
-}
-
 /* flush out the buffers */
 inline int 
 fdflush(const char *filename) 
@@ -923,3 +1081,177 @@ wpadit (char *dest, const char *source, size_t width)
     *q = '\0';
 }
 #endif
+
+void
+frob_lib_modules (void)
+{
+  struct stat statbuf;
+  char buf[PATH_MAX];
+
+  /*
+   * make a symlink from /lib/modules to /target/lib/modules
+   * so that 'depmod' will function properly
+   */
+
+  strncpy(buf, "/lib/modules", sizeof(buf));
+
+  if (! NAME_ISDIR("/target/lib/modules", &statbuf)) {
+    INFOMSG("can't make symlink from /lib/modules to /target/lib/modules, the latter does not exist");
+    return;
+  }
+  if (NAME_ISDIR(buf, &statbuf) ) {
+    snprintf(prtbuf, sizeof(prtbuf), "%s.old", buf);
+    INFOMSG("moving away %s to %s", buf, prtbuf);
+    /* deal with existing /lib/modules.old, rather imperiously */
+    execlog("rm -rf /lib/modules.old", LOG_DEBUG);/* uh, I should use prtbuf bug I'm lazy */
+    rename(buf, prtbuf);
+  } else {
+    DEBUGMSG("non-directory %s, unlinking", buf)
+    unlink(buf);
+  }
+  INFOMSG("making %s a link to /target/lib/modules", buf);
+  symlink("/target/lib/modules", buf);
+
+}
+
+int
+max (int a, int b)
+{
+    return a > b ? a : b;
+}
+
+#define GFST_ext2 "ext2"
+#define GFST_ext3 "ext3"
+#define GFST_reiserfs "reiserfs"
+#define GFST_xfs "xfs"
+
+const char *
+get_fs_type(const char *name)
+{
+	FILE *fd;
+	char xfsmagic[6];
+	char e2fsmagic[4];
+	char shfsmagic[11];
+
+	bzero(xfsmagic, sizeof(xfsmagic));
+	bzero(e2fsmagic, sizeof(e2fsmagic));
+	bzero(shfsmagic, sizeof(shfsmagic));
+	fd = fopen(name, "r");
+	if (!fd)
+		return NULL;
+
+	fread (xfsmagic, 5, 1, fd);
+	fseek (fd, 65588, SEEK_SET);
+	fread (shfsmagic, 10, 1, fd);
+	fseek (fd, 1116, SEEK_SET);
+	fread (e2fsmagic, 1, 4, fd);
+
+
+
+	if (is_xfs && (!(strcmp(xfsmagic, "XFSB")))) /* XFS */
+		return GFST_xfs;
+	else if (is_reiserfs && (!(strcmp(shfsmagic, "ReIsErFs") && strcmp(shfsmagic, "ReIsEr2Fs")))) /* reiserfs */
+		return GFST_reiserfs;
+	else if (is_ext3 && (*e2fsmagic & 0x0004)) /* ext3 */
+		return GFST_ext3;
+
+	return GFST_ext2;
+}
+
+
+/*
+ * "/proc/sys/dev/cdrom/info", on bittersweet, contains:
+ * 8<------------------------------------------------->8
+ * CD-ROM information, Id: cdrom.c 2.56 1999/09/09
+ *
+ * drive name:             sr0     hdd
+ * drive speed:            6       4
+ * drive # of slots:       1       0
+ * Can close tray:         1       1
+ * Can open tray:          1       1
+ * Can lock tray:          1       1
+ * Can change speed:       1       1
+ * Can select disk:        0       0
+ * Can read multisession:  1       1
+ * Can read MCN:           1       1
+ * Reports media changed:  1       1
+ * Can play audio:         1       1
+ *
+ *
+ * 8<------------------------------------------------->8
+ *
+ * Note that there are actually tab characters in the file; I've
+ * untabified so it stays lined up in this comment.  I have verified
+ * that this file does not exist on a machine with no CDROM attached.
+ * - karlheg 2000.03.20
+ * Linux bittersweet 2.2.14 #9 Fri Mar 17 00:50:19 PST 2000 i586 unknown
+ */
+
+int
+have_cdrom ()
+{
+    FILE *cd_info;
+    char *line = NULL;
+    size_t line_size = 0;
+    if ((cd_info = fopen ("/proc/sys/dev/cdrom/info", "r")) != NULL) {
+       /* Read the file and build a list of available CD devices. */
+       getline (&line, &line_size, cd_info);
+       getline (&line, &line_size, cd_info);
+       getline (&line, &line_size, cd_info);
+       if(strstr(line, "\t\t") != NULL) {
+          /* leading tabs exist so the device names must follow in the line */
+          fclose(cd_info);
+          return 1;
+       }
+       fclose(cd_info);
+    }
+    return 0;
+}
+
+#ifdef USE_LANGUAGE_CHOOSER
+void find_opt_server (char * hostname)
+{
+   if (NULL != lang && NULL != lang->msgcat && NULL != lang->locale)
+   {
+      int i=0;
+      if(strlen(lang->locale) < 5)
+         goto otherway;
+      while(mirror_list[i] != NULL) {
+         if(strncasecmp((lang->locale)+3,mirror_list[i], 2) == 0) {
+            sprintf(hostname, "ftp.%s.debian.org", mirror_list[i]);
+            return;
+         }
+         else
+            i++;
+      }
+otherway:
+      /* okay, no country matching, trying the another mapping */
+      i=0;
+      while(mirror_country_map[i] != NULL) {
+         if(strcmp(lang->msgcat,mirror_country_map[i]) == 0) {
+            sprintf(hostname, "ftp.%s.debian.org", mirror_country_map[i+1]);
+            return;
+         }
+         else
+            i+=2;
+      }
+   }
+}
+#endif
+
+char *
+concat_paths (const char *path1, const char *path2)
+{
+    char *path = malloc(strlen(path1) + 1 + strlen(path2) + 1);
+    size_t len;
+
+    strcpy(path, path1);
+
+    len = strlen (path1);
+    if (len > 0 && path1[len-1] != '/' && path2[0] != '/')
+	path[len++] = '/';
+
+    strcpy(path + len, path2);
+
+    return path;
+}
