@@ -39,8 +39,8 @@ writeFloppy(
 	n = totalSize - (bytesPerFloppy * (floppyNumber - 1));
 	if ( n < bytesToWrite ) {
 		bytesToWrite = n;
-		if ( bytesToWrite % 512 > 0 )
-			pad = 1024 - (bytesToWrite % 512);
+		if ( bytesPerFloppy - bytesToWrite > 0 )
+			pad = (bytesPerFloppy - bytesToWrite);
 	}
 
 	if ( t == 0 ) {
@@ -97,7 +97,7 @@ writeFloppy(
 		bytesToWrite -= chunk;
 	}
 	if ( pad > 0 ) {
-		char	buffer[512];
+		char	buffer[pad];
 		memset(buffer, 0, sizeof(buffer)); 
 		write(outputFd, buffer, pad);
 	}
