@@ -32,7 +32,7 @@
  * not contain this for some reason -- Ben
  */
 #ifndef MAX_DISKNAME_LEN
-#define MAX_DISKNAME_LEN 32
+#define MAX_DISKNAME_LEN PATH_MAX
 #endif
 
 /* this array translates partition type numbers to file system types */
@@ -46,6 +46,7 @@ static struct fstypes {
     { PTYPE_DOS16S,		FSTYPE_MSDOS },
     { PTYPE_DOSEXT,		FSTYPE_EXTPART },
     { PTYPE_DOS16L,		FSTYPE_MSDOS },
+    { PTYPE_NTFS, 		FSTYPE_MSDOS },
     { PTYPE_WFAT32,		FSTYPE_MSDOS },
     { PTYPE_WFAT32L,		FSTYPE_MSDOS },
     { PTYPE_WEXT,		FSTYPE_EXTPART },
@@ -53,6 +54,8 @@ static struct fstypes {
     { PTYPE_LSWAP,		FSTYPE_SWAP },
     { PTYPE_LINUX,		FSTYPE_EXT2 },
     { PTYPE_LINUXEXT,		FSTYPE_EXTPART },
+    { PTYPE_GPT,		FSTYPE_UNKNOWN },
+    { PTYPE_EFI,		FSTYPE_MSDOS },
 #endif /* HAVE_MSDOS_PARTITION */
 #if HAVE_BSD_DISKLABEL
     { PTYPE_BSD_MSDOS,		FSTYPE_MSDOS },
@@ -126,6 +129,17 @@ static struct fstypes {
     { PTYPE_ACORN_ICS_EXT2,	FSTYPE_EXT2 },
     { PTYPE_ACORN_ICS_SWAP,	FSTYPE_SWAP },
 #endif /* HAVE_ACORN_PARTITION */
+#if HAVE_SGI_PARTITION
+    { PTYPE_SGI_L_SWAP,         FSTYPE_SWAP },
+    { PTYPE_SGI_L_NATIVE,       FSTYPE_EXT2 },
+    { PTYPE_SGI_VOLHDR,         FSTYPE_UNKNOWN },
+    { PTYPE_SGI_VOL,            FSTYPE_UNKNOWN },
+#endif /* HAVE_SGI_PARTITION */
+#if HAVE_IBM_PARTITION
+    { PTYPE_IBM_L_SWAP,         FSTYPE_SWAP },
+    { PTYPE_IBM_L_NATIVE,       FSTYPE_EXT2 },
+    { PTYPE_IBM_LINUX,          FSTYPE_UNKNOWN },
+#endif /* HAVE_IBM_PARTITION */
 #if HAVE_LOOP_PARTITION
     /* Fake partition types for loop mounts */
     { PTYPE_LOOP_EXT2,		FSTYPE_EXT2 },
@@ -151,10 +165,10 @@ static struct systypes {
 #if HAVE_MSDOS_PARTITION
     /* PC */
     {0, "Empty"},
-    {1, "DOS 12-bit FAT"},              /* Primary DOS with 12-bit FAT */
-    {2, "XENIX /"},                     /* XENIX / filesystem */
-    {3, "XENIX /usr"},                  /* XENIX /usr filesystem */
-    {4, "DOS 16-bit FAT <32M"},         /* Primary DOS with 16-bit FAT */
+    {1, "DOS 12-bit FAT"},
+    {2, "XENIX root"},
+    {3, "XENIX usr"},
+    {4, "DOS 16-bit FAT <32M"},
     {5, "DOS Extended"},                /* DOS 3.3+ extended partition */
     {6, "DOS 16-bit FAT >=32M"},
     {7, "HPFS / NTFS"},
@@ -168,60 +182,75 @@ static struct systypes {
     {0x10, "OPUS"},
     {0x11, "Hidden DOS FAT12"},
     {0x12, "Compaq diagnostics"},
-    {0x14, "Hidden DOS FAT16"},
-    {0x16, "Hidden DOS FAT16 (big)"},
+    {0x14, "Hidden FAT16 <32M"},
+    {0x16, "Hidden FAT16"},
     {0x17, "Hidden HPFS/NTFS"},
-    {0x18, "AST Windows swapfile"},
+    {0x18, "AST Windows SmartSleep"},
+    {0x1b, "Hidden Win95 FAT32"},
+    {0x1c, "Hidden Win95 FAT32 (LBA)"},
+    {0x1e, "Hidden Win95 FAT16 (LBA)"},
     {0x24, "NEC DOS"},
+    {0x39, "Plan 9"},
     {0x3c, "PartitionMagic recovery"},
     {0x40, "Venix 80286"},
-    {0x41, "Linux/MINIX (sharing disk with DRDOS)"},
-    {0x42, "SFS or Linux swap (sharing disk with DRDOS)"},
+    {0x41, "PPC PReP Boot"},
+    {0x42, "Windows 2000 Dynamic Partition (Formerly SFS)"},
     {0x43, "Linux native (sharing disk with DRDOS)"},
-    {0x50, "DM (disk manager)"},
-    {0x51, "DM6 Aux1 (or Novell)"},
+    {0x4d, "QNX4.x"},
+    {0x4e, "QNX4.x 2nd part"},
+    {0x4f, "QNX4.x 3rd part"},
+    {0x50, "OnTrack Disk Manager"},
+    {0x51, "OnTrack DM6 Aux1 (or novell)"},
     {0x52, "CP/M or Microport SysV/AT"},
-    {0x53, "DM6 Aux3"},
-    {0x54, "DM6"},
-    {0x55, "EZ-Drive (disk manager)"},
-    {0x56, "Golden Bow (disk manager)"},
-    {0x5c, "Priam Edisk (disk manager)"},
+    {0x53, "OnTrack DM6 Aux3"},
+    {0x54, "OnTrack DM6"},
+    {0x55, "EZ-Drive disk manager"},
+    {0x56, "Golden Bow disk manager"},
+    {0x5c, "Priam Edisk disk manager"},
     {0x61, "SpeedStor"},
     {0x63, "GNU HURD or Mach or Sys V/386 (such as ISC UNIX)"},
     {0x64, "Novell Netware 286"},
     {0x65, "Novell Netware 386"},
     {0x70, "DiskSecure Multi-Boot"},
     {0x75, "PC/IX"},
-    {0x77, "QNX4.x"},
-    {0x78, "QNX4.x 2nd part"},
-    {0x79, "QNX4.x 3rd part"},
-    {0x80, "MINIX until 1.4a"},
-    {0x81, "MINIX / old Linux"},
+    {0x80, "MINIX prior to 1.4a"},
+    {0x81, "MINIX 1.4b or later / old Linux"},
     {0x82, "Linux swap"},
     {0x83, "Linux native"},
     {0x84, "OS/2 hidden C: drive"},
     {0x85, "Linux extended"},
     {0x86, "NTFS volume set"},
     {0x87, "NTFS volume set"},
+    {0x8e, "Linux LVM"},
     {0x93, "Amoeba"},
-    {0x94, "Amoeba BBT"},               /* (bad block table) */
+    {0x94, "Amoeba Bad Block Table"},               /* (bad block table) */
+    {0x9f, "BSD/OS"},		/* BSDI */
     {0xa0, "IBM Thinkpad hibernation"}, /* according to dan@fch.wimsey.bc.ca */
     {0xa5, "BSD/386"},                  /* 386BSD */
-    {0xa7, "NeXTSTEP 486"},
+    {0xa6, "OpenBSD"},
+    {0xa7, "NeXTSTEP"},
     {0xb7, "BSDI fs"},
     {0xb8, "BSDI swap"},
+    {0xbb, "Boot Wizard hidden"},
     {0xc1, "DRDOS/sec (FAT-12)"},
-    {0xc4, "DRDOS/sec (FAT-16, < 32M)"},
-    {0xc6, "DRDOS/sec (FAT-16, >= 32M)"},
+    {0xc4, "DRDOS/sec (FAT-16) < 32M"},
+    {0xc6, "DRDOS/sec (FAT-16)"},
     {0xc7, "Syrinx"},
+    {0xda, "Non-FS data"},
     {0xdb, "CP/M or Concurrent CP/M or Concurrent DOS or CTOS"},
+    {0xde, "Dell Utility"},
+    {0xdf, "BootIt"},
     {0xe1, "DOS access or SpeedStor 12-bit FAT extended partition"},
     {0xe3, "DOS R/O or SpeedStor"},
     {0xe4, "SpeedStor 16-bit FAT extended partition < 1024 cyl."},
+    {0xeb, "BeOS fs"},
+    {0xee, "Intel EFI GUID Partition Table"},
+    {0xef, "Intel EFI System Partition (FAT-12/16/32)"},
+    {0xf0, "Linux/PA-RISC boot"},
     {0xf1, "SpeedStor"},
     {0xf2, "DOS 3.3+ secondary"},
     {0xf4, "SpeedStor large partition"},
-    {0xfd, "Linux RAID autodetect"},
+    {0xfd, "Linux RAID (autodetect)"},
     {0xfe, "SpeedStor >1024 cyl. or LANstep"},
     {0xff, "Xenix Bad Block Table"},
 #endif /* HAVE_MSDOS_PARTITION */
@@ -319,11 +348,18 @@ static struct systypes {
 #endif /* HAVE_ATARI_PARTITION */
 #if HAVE_MAC_PARTITION
     { PTYPE_MAC_PMAP,		"partition map" },
-    { PTYPE_MAC_DRIVER,		"Driver" },
-    { PTYPE_MAC_DRIVER43,	"Driver 4.3" },
+    { PTYPE_MAC_DRIVER,		"MacOS Driver" },
+    { PTYPE_MAC_DRIVER43,	"MacOS Driver 4.3" },
+    { PTYPE_MAC_ATADRIVER,      "MacOS ATA Driver" },
+    { PTYPE_MAC_FWDRIVER,       "MacOS FW Driver" },
+    { PTYPE_MAC_IODRIVER,       "MacOS IOKit" },
+    { PTYPE_MAC_PATCH,          "MacOS Patches" },
+    { PTYPE_MAC_OSXBOOT,        "MacOSX bootloader" },
+    { PTYPE_MAC_OSXLOADER,      "MacOSX loader" },
+    { PTYPE_MAC_UFS,            "UFS" },
     { PTYPE_MAC_HFS,		"HFS" },
     { PTYPE_MAC_MFS,		"MFS" },
-    { PTYPE_MAC_SCRATCH,	"Scratch (swap)" },
+    { PTYPE_MAC_SCRATCH,	"Scratch" },
     { PTYPE_MAC_PRODOS,		"ProDOS" },
     { PTYPE_MAC_FREE,		"Free" },
     { PTYPE_MAC_SWAP,		"Linux swap" },
@@ -332,6 +368,7 @@ static struct systypes {
     { PTYPE_MAC_MINIX,		"Minix" },
     { PTYPE_MAC_AFFS,		"Amiga AFFS" },
     { PTYPE_MAC_EXT2,		"Linux native" },
+    { PTYPE_MAC_BOOT,		"NewWorld bootblock" },
 #endif /* HAVE_MAC_PARTITION */
 #if HAVE_ACORN_PARTITION
     { PTYPE_ACORN_LINUX,	"ARM Linux Partition Map" },
@@ -342,6 +379,28 @@ static struct systypes {
     { PTYPE_ACORN_ICS_EXT2,	"Linux ext2 (ICS/APDL)" },
     { PTYPE_ACORN_ICS_SWAP,	"Linux swap (ICS/APDL)" },
 #endif /* HAVE_ACORN_PARTITION */
+#if HAVE_SGI_PARTITION
+    { PTYPE_SGI_VOLHDR,          "SGI volhdr" },
+    { PTYPE_SGI_TREPL,           "SGI trkrepl" },
+    { PTYPE_SGI_SREPL,           "SGI secrepl" },   
+    { PTYPE_SGI_SWAP,            "SGI swap" },    
+    { PTYPE_SGI_BSD,             "BSD " },   
+    { PTYPE_SGI_SYSV,            "System V" },
+    { PTYPE_SGI_VOL,             "SGI disk" },
+    { PTYPE_SGI_EFS,             "SGI efs" },
+    { PTYPE_SGI_LVOL,            "SGI lvol" },
+    { PTYPE_SGI_RLVOL,           "SGI rlvol" },
+    { PTYPE_SGI_XFS,             "SGI xfs" },
+    { PTYPE_SGI_XLVOL,           "SGI xlvol" },
+    { PTYPE_SGI_RXLVOL,          "SGI rxlvol" },
+    { PTYPE_SGI_L_SWAP,          "Linux Swap" },
+    { PTYPE_SGI_L_NATIVE,        "Linux native" },
+#endif /* HAVE_SGI_PARTITION */
+#if HAVE_IBM_PARTITION
+    { PTYPE_IBM_L_SWAP,          "Linux Swap" },
+    { PTYPE_IBM_L_NATIVE,        "Linux native" },
+    { PTYPE_IBM_LINUX,           "Linux native or swap" },
+#endif /* HAVE_IBM_PARTITION */
 #if HAVE_LOOP_PARTITION 
     { PTYPE_LOOP_EXT2,		"Linux native (loop mounted)" },
     { PTYPE_LOOP_SWAP,		"Linux swap (loop mounted)" },
@@ -356,6 +415,7 @@ static struct systypes {
 
 char *fstype_name[FSTYPE_MAX] = {
     "ext2",
+    "ext2", /* this will have to be changed when (if?) ReiserFS gets its own partition type */
     "swap",
     "msdos",
     "minix",
@@ -472,7 +532,7 @@ fdisk_fstype_name_of(unsigned int ptype)
 /* get_part_info: read and parse line in /proc/partitions format */
 static int get_part_info(FILE *f, dev_t *dev, char *name)
 {
-    char line[64];
+    char line[PATH_MAX];
     unsigned int major, minor;
 
     while (fgets(line, sizeof line, f)) {
@@ -509,6 +569,9 @@ static struct DEVICE_BLOCK {
     { 57 << 8,   2, 64 },	/* hdk      - hdl       */
     { 65 << 8, 112, 16 },	/* sdq      - sddx      */
     { 72 << 8, 128, 16 },	/* ida/c0d0 - ida/c7d15 */
+    { 94 << 8,  26,  4 },	/* dasda    - dasdz     */
+    {104 << 8, 256, 16 },       /* cciss/c0d0 - cciss/c7d15 */
+    {114 << 8,   8, 16 },       /* ataraid/d0 - ataraid/d7 */
     {       0,   0,  0 }
 };
 
@@ -541,7 +604,7 @@ static char *strdupcat(char *s1, char *s2)
 
 
 /* part_name: return partition device name from disk and partition number */
-static char *part_name(struct fdisk_disk *disk, unsigned int part)
+char *part_name(struct fdisk_disk *disk, unsigned int part)
 {
     FILE *f;
     char *pname = NULL, *dname = disk->name + 5;
@@ -678,6 +741,7 @@ fdisk_find_partition_by_mntpoint(const char* mount_point)
   struct fdisk_partition* p;
 
   if (mount_point) {
+    mounted_reread();
     p = mounted_partitions;
 
     while (p) {
@@ -740,11 +804,12 @@ mounted_reread(){
     fprintf(stderr,"mounted partition: %s, on: %s, type: %s , options: %s ... ", 
 	mnt_ent->mnt_fsname, mnt_ent->mnt_dir, mnt_ent->mnt_type, mnt_ent->mnt_opts);
 #endif
-    /* If mnt_ent is proc or devpts, ignore it */
+    /* If mnt_ent is proc, devfs or devpts, ignore it */
     if ( (strcmp(mnt_ent->mnt_type,"proc") == 0) ||
+         (strcmp(mnt_ent->mnt_type,"devfs") == 0) ||
 	 (strcmp(mnt_ent->mnt_type,"devpts") == 0) ) {
 #ifdef DEBUG
-      fprintf(stderr,"proc/devpts fs. Ignoring it.\n");
+      fprintf(stderr,"proc/devfs/devpts fs. Ignoring it.\n");
 #endif
       continue;
     }
@@ -1021,13 +1086,37 @@ static struct {
     char *name;
     int (*func)(char *device, int fd);
 } parsers[] = {
+/* Put GPT before MSDOS, as GPT lives inside an msdos type EE partition */
+#ifdef HAVE_GPT_PARTITION
+    { "gpt", parse_gpt_partition },
+#endif
+#ifdef HAVE_MSDOS_PARTITION
     { "msdos", parse_msdos_partition },
+#endif
+#ifdef HAVE_OSF_PARTITION
     { "osf",   parse_osf_partition },
+#endif
+#ifdef HAVE_SUN_PARTITION
     { "sun",   parse_sun_partition },
+#endif
+#ifdef HAVE_AMIGA_PARTITION
     { "amiga", parse_amiga_partition },
+#endif
+#ifdef HAVE_ATARI_PARTITION
     { "atari", parse_atari_partition },
+#endif
+#ifdef HAVE_MAC_PARTITION
     { "mac",   parse_mac_partition },
-    { "acorn", parse_acorn_partition }
+#endif
+#ifdef HAVE_ACORN_PARTITION
+    { "acorn", parse_acorn_partition },
+#endif
+#ifdef HAVE_SGI_PARTITION
+    { "sgi",   parse_sgi_partition },
+#endif
+#ifdef HAVE_IBM_PARTITION
+    { "ibm",   parse_ibm_partition },
+#endif
 };
 
 static void
@@ -1095,6 +1184,10 @@ static int is_disk(char *device, int fd)
     if (SCSI_DISK_MAJOR(MAJOR(stat_buf.st_rdev))) 
 	return 1;
 
+    /* DASD disk driver will only accept disks */
+    if (DASD_MAJOR == MAJOR(stat_buf.st_rdev)) 
+	return 1;
+
 #if #cpu (i386)
     /* ESDI devices assumed to all be disks */
     if (MAJOR(stat_buf.st_rdev) == PS2ESDI_MAJOR)
@@ -1184,7 +1277,7 @@ void fdisk_reread(void)
 	    /* check device number against known 'whole-disk' devices */
 	    if (is_drive(dev)) {
 		static char path[] = "/dev/";
-		char dev_name[sizeof path + sizeof name];
+		char dev_name[PATH_MAX];
 
 		/* try to build partition list for device */
 		strcpy(dev_name, path);
