@@ -35,16 +35,16 @@ struct fdisk_partition {
 struct fdisk_disk *fdisk_disks;
 struct fdisk_partition *fdisk_partitions;
 #define FSTYPE_EXT2	0
-#define FSTYPE_SWAP	1
-#define FSTYPE_MSDOS	2
-#define FSTYPE_MINIX	3
-#define FSTYPE_AFFS	4
-#define FSTYPE_HFS	5
-#define FSTYPE_ADFS	6
-#define FSTYPE_NFS	7
-#define FSTYPE_EXTPART	8
-#define FSTYPE_UNKNOWN	9
-#define FSTYPE_MAX	10
+#define FSTYPE_SWAP	2
+#define FSTYPE_MSDOS	3
+#define FSTYPE_MINIX	4
+#define FSTYPE_AFFS	5
+#define FSTYPE_HFS	6
+#define FSTYPE_ADFS	7
+#define FSTYPE_NFS	8
+#define FSTYPE_EXTPART	9
+#define FSTYPE_UNKNOWN	10
+#define FSTYPE_MAX	11
 struct fdisk_partition *fdisk_partitions_by_type[FSTYPE_MAX];
 /* points to the list of partitions, index is FSTYPE_* */
 struct fdisk_partition *mounted_partitions;
@@ -79,6 +79,8 @@ fdisk_add_disk(const char *name, unsigned long size);
 int
 fdisk_fstype_of(unsigned int ptype);
 
+char *part_name(struct fdisk_disk *disk, unsigned int part);
+
 char *
 fdisk_fstype_name_of(unsigned int ptype);
 
@@ -89,6 +91,9 @@ fdisk_add_partition(const char *name, int minor, unsigned int type,
 int
 sread(int fd, unsigned long sector, void *buffer);
 
+#if HAVE_GPT_PARTITION
+int parse_gpt_partition(char *device, int fd);
+#endif
 #if HAVE_MSDOS_PARTITION
 int parse_msdos_partition(char *device, int fd);
 #endif
@@ -110,6 +115,13 @@ int parse_mac_partition(char *device, int fd);
 #if HAVE_ACORN_PARTITION
 int parse_acorn_partition(char *device, int fd);
 #endif
+#if HAVE_SGI_PARTITION
+int parse_sgi_partition(char *device, int fd);
+#endif
+#if HAVE_IBM_PARTITION
+int parse_ibm_partition(char *device, int fd);
+#endif
+
 
 /* -------------------- partition type numbers -------------------- */
 /*
@@ -130,12 +142,14 @@ int parse_acorn_partition(char *device, int fd);
 #define PTYPE_UNKNOWN		0x00000100
 
 /* PC */
+/* i really wish that they would make a real number for reiser */
 #define PTYPE_PREFIX_MSDOS	0x00000000
 #define PTYPE_EMPTY		0
 #define PTYPE_DOS12		0x01
 #define PTYPE_DOS16S		0x04
 #define PTYPE_DOSEXT		0x05
 #define PTYPE_DOS16L		0x06
+#define PTYPE_NTFS		0x07
 #define PTYPE_WFAT32		0x0b
 #define PTYPE_WFAT32L		0x0c
 #define PTYPE_WEXT		0x0f
@@ -145,11 +159,17 @@ int parse_acorn_partition(char *device, int fd);
 #define PTYPE_MINIX		0x81
 #define PTYPE_LSWAP		0x82
 #define PTYPE_LINUX		0x83
+#define PTYPE_REISERFS		0x83
 #define PTYPE_LINUXEXT		0x85
+#define PTYPE_GPT		0xee
+#define PTYPE_EFI		0xef
 
 /* PowerPC PReP */
 #define PTYPE_PREP_BOOT		0x41
 
+/* PowerPC CHRP */
+#define PTYPE_CHRP_BOOT		0x41
+
 /* BSD disklabel (contained in MSDOS-partition); uses codes 0x425344xx
  * ("BSD"), where xx is the 8 bit type */
 #define PTYPE_PREFIX_BSD	0x42534400
@@ -273,6 +293,14 @@ int parse_acorn_partition(char *device, int fd);
 #define PTYPE_MAC_MINIX		(PTYPE_PREFIX_MAC|0x0c)
 #define PTYPE_MAC_AFFS		(PTYPE_PREFIX_MAC|0x0d)
 #define PTYPE_MAC_EXT2		(PTYPE_PREFIX_MAC|0x0e)
+#define PTYPE_MAC_BOOT 		(PTYPE_PREFIX_MAC|0x0f)
+#define PTYPE_MAC_ATADRIVER     (PTYPE_PREFIX_MAC|0x10)
+#define PTYPE_MAC_FWDRIVER      (PTYPE_PREFIX_MAC|0x11)
+#define PTYPE_MAC_IODRIVER      (PTYPE_PREFIX_MAC|0x12)
+#define PTYPE_MAC_PATCH         (PTYPE_PREFIX_MAC|0x13)
+#define PTYPE_MAC_OSXBOOT       (PTYPE_PREFIX_MAC|0x14)
+#define PTYPE_MAC_OSXLOADER     (PTYPE_PREFIX_MAC|0x15)
+#define PTYPE_MAC_UFS           (PTYPE_PREFIX_MAC|0x16)
 
 /* Acorn; uses codes 0x41524dxx ("ARM"), where xx is a number.
    These numbers are synthetic, and are for indentification in
@@ -286,6 +314,28 @@ int parse_acorn_partition(char *device, int fd);
 #define PTYPE_ACORN_ICS_EXT2	(PTYPE_PREFIX_ACORN|0x06)
 #define PTYPE_ACORN_ICS_SWAP	(PTYPE_PREFIX_ACORN|0x07)
 
+/* SGI partition ids */
+#define PTYPE_SGI_VOLHDR   0x00  /* volhdr */
+#define PTYPE_SGI_TREPL    0x01 /* trkrep */
+#define PTYPE_SGI_SREPL	   0x02 /* secrepl */
+#define PTYPE_SGI_SWAP     0x03 /* swap   */
+#define PTYPE_SGI_BSD      0x04 /* bsd    */
+#define PTYPE_SGI_SYSV     0x05 /* sysv   */
+#define PTYPE_SGI_VOL      0x06 /* entire disk/volume */
+#define PTYPE_SGI_EFS      0x07 /* efs    */
+#define PTYPE_SGI_LVOL     0x08 /* lvol   */
+#define PTYPE_SGI_RLVOL    0x09 /* rlvol  */
+#define PTYPE_SGI_XFS	   0x0A /* xfs    */
+#define PTYPE_SGI_XLVOL	   0x0B /* xlvo   */
+#define PTYPE_SGI_RXLVOL   0x0C /* rxlvol */
+#define PTYPE_SGI_L_SWAP   0x82
+#define PTYPE_SGI_L_NATIVE 0x83
+
+/* IBM partition ids */
+#define PTYPE_IBM_L_SWAP   0x82
+#define PTYPE_IBM_L_NATIVE 0x83
+#define PTYPE_IBM_LINUX    0x84
+
 /* Fake partition types for loop mounts */
 #define PTYPE_PREFIX_LOOP	0x4c4f5000
 #define PTYPE_LOOP_EXT2		(PTYPE_PREFIX_LOOP|FSTYPE_EXT2)
