@@ -28,7 +28,7 @@ int main() {
 
   p = fdisk_partitions;
   while (p) {
-    printf("%s (%s), ", p->name, sysname(p->type));
+    printf("%s (%s), ", p->name, fdisk_sysname(p->type));
     p = p->next;
   }
   printf("\n");
